//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function qdd = accel(robot, Q, qd, torque)
// Manipulator forward dynamics
//
// Calling Sequence
// QDD = accel(R, Q, QD, TORQUE)
// QDD = accel(R,X) 
//
// Description
// <literal>QDD = accel(R, Q, QD, TORQUE)</literal> is a vector (Nx1) of joint accelerations that result 
// from applying the actuator force/torque to the manipulator robot in state Q and QD.
// If Q, QD, TORQUE are matrices (KxN) then QDD is a matrix (KxN) where each row 
// is the acceleration corresponding to the equivalent rows of Q, QD, TORQUE.
//
// <literal>QDD = accel(R, X)</literal> as above but X=[Q,QD,TORQUE]
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>Uses the method 1 of Walker and Orin to compute the forward dynamics.</para></listitem>
// <listitem><para>This form is useful for simulation of manipulator dynamics, in
//   conjunction with a numerical integration function.</para></listitem>
// </itemizedlist>
// </refsection>
//
// Bibliography
// [1] Efficient dynamic computer simulation of robotic mechanisms, M. W. Walker and D. E. Orin,
// [2] ASME Journa of Dynamic Systems, Measurement and Control, vol. 104, no. 3, pp. 205-211, 1982.
//
// See also 
// rne
// SerialLink
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014 - Davide Cappucci

    [%nargout, %nargin] = argn(0);
    
	n = robot.n;

    if %nargin == 2
        if length(Q) ~= (3*robot.n)
            error(msprintf(gettext('RTB:accel:badarg', 'Input vector X is length %d, should be %d (q, qd, tau)'), length(Q), 3*robot.n));
        end
        // accel(X)
        Q = Q(:).';   // make it a row vector
        q = Q(1:n);
		qd = Q(n+1:2*n);
		torque = Q(2*n+1:3*n);

    elseif %nargin == 4
        // accel(robot, Q, qd, torque)
        
        if numrows(Q) > 1
            // handle trajectory by recursion
            if numrows(Q) ~= numrows(qd) then
                error('for trajectory q and qd must have same number of rows');
            end
            if numrows(Q) ~= numrows(torque) then
                error('for trajectory q and torque must have same number of rows');
            end
            qdd = [];
            for i=1:numrows(Q)
                qdd = cat(1, qdd, accel(robot, Q(i,:), qd(i,:), torque(i,:))');
            end
            return
        else
            q = Q';
            if length(q) == n then
                q = q(:)';
                qd = qd(:)';
            end
            if numcols(Q) ~= n then
                error('q must have %d columns', n);
            end
            if numcols(qd) ~= robot.n then
                error('qd must have %d columns', n);
            end
            if numcols(torque) ~= robot.n then
                error('torque must have %d columns', n);
            end
        end
    else
        error('accel:badargs    insufficient arguments');
        break;
    end

	// compute current manipulator inertia
	//   torques resulting from unit acceleration of each joint with
	//   no gravity.
	M = frne(robot, ones(n,1)*q, zeros(n,n), eye(n,n), [0;0;0]);
	// compute gravity and coriolis torque
	//    torques resulting from zero acceleration at given velocity &
	//    with gravity acting.
	tau = frne(robot, q, qd, zeros(1,n));	

	qdd = M \ (torque - tau)';

endfunction
