//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [x, y, typ] = accel_if(job, arg1, arg2)
	// accel_if
	//
	// Description
	// This block computes a vector of joint accelerations that result from applying the actuator torque to the manipulator SerialLink with joint coordinates q and velocities qd.
	//
	// The number of input of this block is three. The first input represents the joint state vector q.
	// The second input is the joint velocity qd. The third input represents the vector torque of generalized torques applied to manipulator's end-effector.
	// Each of these vector valued signals must have n elements, where n is the number of SerialLink's constituent links.
	// 
	// This block has one regular output, the joint acceleration vector.
	//
	// The robot model SerialLink can be specified as block parameter.
	//
	// <refsection>
    // <title>Dialog Box</title>
    // <variablelist>
    //   <varlistentry><term>Robot object    :</term>
    // 	<listitem><para>SerialLink. It represents the robot model to be simulated. Note that this variable is a symbolic parameter which has to be defined in the context of the Xcos diagram in order to be used.</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
	//
	// See also
	// robot_if
	// rne_if
	// accel
	// 
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci
	
    x = [];
    y = [];
    typ = [];
	style = 'blockWithLabel;fillColor=red;displayedLabel=accel;fontStyle=1;fontSize=12';
    displayedLabel=accel
    in_label = ['q';'qd';'torque'];
	out_label = ['qdd'];
    select job,
        case "set" then
            x = arg1;
            graphics = arg1.graphics;
            exprs = graphics.exprs;
            model = arg1.model;
            while %T do,
                [ok, robotName, exprs] = getvalue(..
                    "Compute robot joint accelerations",..
                    ["Robot object"],..
                    list("str", -1),..
                    exprs);
                    robot = evstr(robotName);
                if ok & typeof(robot) ~= "Robot" then
                    // not a robot
                    message("invalid block parameter: required a robot object");
                    ok = %F;
                elseif ok & robot.n == 0 then
                    // a robot model to be simulated still must be specified
                    ok = %F;
                end
                if ~ok then
                    break;
                end
                [model, graphics, ok] = set_io(model, graphics, list([robot.n, 1; robot.n, 1; robot.n, 1], [1; 1; 1]), list([robot.n, 1], 1), [], []);
                if ok then //list(robot.dyn, robot.gravity, 
                    model.opar = list(robot.links, robot.gravity, robot.offset, robot.mdh);
                    graphics.exprs = exprs;
					graphics.style = style;
                    graphics.gr_i(1) = "xstringb(orig(1), orig(2), [""accel""], sz(1), sz(2), ""fill"");";
                    x.graphics = graphics;
                    x.model = model;
                    break
                end
            end

        case "define" then
            model = scicos_model();
            model.sim = list("accel_cf", 5);
            model.in = [-1; -1; -1];
            model.in2 = [1; 1; 1];
            model.intyp = [1; 1; 1];
            model.out = -1;
            model.out2 = 1;
            model.outtyp = 1;
            model.opar = list();
            model.blocktype = "d";
            model.dep_ut = [%T %F];
            exprs = "SerialLink()";
            gr_i = ["xstringb(orig(1), orig(2), [""accel""], sz(1), sz(2), ""fill"");"];
            x = standard_define([4 3], model, exprs, gr_i);
			x.graphics.style = style;
			x.graphics.in_label = in_label;
			x.graphics.out_label = out_label;
    end

endfunction
