//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function traj = ctraj(T0, T1, t)
// Cartesian trajectory between two points
//
// Calling Sequence
// TC = CTRAJ(T0, T1, N)
// TC = CTRAJ(T0, T1, S)
//
// Description
// <literal>TC = CTRAJ(T0, T1, N)</literal> is a Cartesian trajectory (4x4xN) from pose T0 to T1
// with N points that follow a trapezoidal velocity profile along the path.
// The Cartesian trajectory is a homogeneous transform sequence and the last 
// subscript being the point index, that is, T(:,:,i) is the i'th point along
// the path.
//
// <literal>TC = CTRAJ(T0, T1, S)</literal> as above but the elements of S (Nx1) specify the 
// fractional distance  along the path, and these values are in the range [0 1].
// The i'th point corresponds to a distance S(i) along the path.
//
// See also 
// lspb
// mstraj
// trinterp
// transl
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014 - Davide Cappucci

    if ~ishomog(T0) | ~ishomog(T1)
        error('arguments must be homogeneous transformations');
    end

    // distance along path is a smooth function of time
    if sum(length(t))== 1 then //isscalar
        s = lspb(0, 1, t);
    else
        s = t(:);
    end

    traj = [];

    for S=s'
        traj = cat(3, traj, trinterp(T0, T1, S));
    end

endfunction
