//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [t, q, qd] = fdyn(robot, t1, torqfun, q0, qd0, varargin)
// Integrate forward dynamics
//
// Calling Sequence
// [T,Q,QD] = fdyn(ROBOT, T, TORQFUN)
// [T,Q,QD] = fdyn(ROBOT, T, TORQFUN, Q0, QD0)
// [T,Q,QD] = fdyn(ROBOT, T, TORQFUN, Q0, QD0, ARG1, ARG2, ...)
//
// Description
// <literal>[T1,Q,QD] = fdyn(ROBOT, T1, TORQFUN)</literal> integrates the dynamics of the robot over 
// the time  interval 0 to T and returns vectors of time TI, joint position Q
// and joint velocity QD.  The initial joint position and velocity are zero.
// The torque applied to the joints is computed by the user function TORQFUN:
//
// <literal>[T1,Q,QD] = fdyn(ROBOT, T1, TORQFUN, Q0, QD0)</literal> as above but allows the initial
// joint position and velocity to be specified.
//
// The control torque is computed by a user defined function
//
//        <literal>TAU = TORQFUN(T, Q, QD, ARG1, ARG2, ...)</literal>
//
// where Q and QD are the manipulator joint coordinate and velocity state 
// respectively, and T is the current time. 
//
// <literal>[T1,Q,QD] = fdyn(ROBOT, T1, TORQFUN, Q0, QD0, ARG1, ARG2, ...)</literal> allows optional 
// arguments to be passed through to the user function.
//
// <refsection>
// <title>Notes</title>
//  <itemizedlist>
//  <listitem><para>This function performs poorly with non-linear joint friction, such as
//   Coulomb friction.  The ROBOT.nofriction() method can be used to set this 
//   friction to zero.</para></listitem>
// <listitem><para>If TORQFUN is not specified, or is given as 0 or [],  then zero torque
//   is applied to the manipulator joints.</para></listitem>
// <listitem><para>The builtin integration function ode() is used.</para></listitem>
//  </itemizedlist>
//	</refsection>
//
// See also 
// accel
// nofriction
// rne
// ode
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014 - Davide Cappucci

    [%nargout, %nargin] = argn(0);

    if %nargin <6 then
        varargin = list();
    end
    n = robot.n;
    if %nargin == 2 then
        torqfun = 0;
        q0 = zeros(1,n);
        qd0 = zeros(1,n);
    elseif %nargin == 3 then
        q0 = zeros(1,n);
        qd0 = zeros(1,n);
    elseif %nargin == 4 then
        qd0 = zeros(1,n);
    end

    // concatenate q and qd into the initial state vector
    q0 = [q0(:); qd0(:)];

    x = ode(q0, 0, t1, list(fdyn2,robot, torqfun, varargin(:)));
    t = t1;
    q = x(1:n,:);
    qd = x(n+1:2*n,:);

endfunction
