//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [qt,histout] = ikine(robot,tr,varargin)
    // Inverse manipulator kinematics
    //
    // Calling Sequence
    // Q = ikine(R, T)
    // Q = ikine(R, T, Q0, OPTIONS)
    // Q = ikine(R, T, Q0, M, OPTIONS)
    //
    // Parameters
    // OPTIONS : key/value
    // 'pinv'        : use pseudo-inverse instead of Jacobian transpose
    // 'ilimit',L    : set the maximum iteration count (default 1000)
    // 'tol',T       : set the tolerance on error norm (default 1e-6)
    // 'alpha',A     : set step size gain (default 1)
    // 'novarstep'   : disable variable step size
    // 'verbose'     : show number of iterations for each point
    // 'verbose=2'   : show state at each iteration
    //  'plot'       :  plot iteration state versus time
    //
    // Description
    // <literal>Q = ikine(R,T)</literal> is the joint coordinates corresponding to the robot 
    // end-effector pose T which is a homogenenous transform.
    //
    // <literal>Q = ikine(R,T, Q0, OPTIONS)</literal> specifies the initial estimate of the joint 
    // coordinates.
    //
    // <literal>Q = ikine(R, T, Q0, M, OPTIONS)</literal> specifies the initial estimate of the joint 
    // coordinates and a mask matrix.  For the case where the manipulator 
    // has fewer than 6 DOF the solution space has more dimensions than can
    // be spanned by the manipulator joint coordinates.  In this case
    // the mask matrix M specifies the Cartesian DOF (in the wrist coordinate 
    // frame) that will be ignored in reaching a solution.  The mask matrix 
    // has six elements that correspond to translation in X, Y and Z, and rotation 
    // about X, Y and Z respectively.  The value should be 0 (for ignore) or 1.
    // The number of non-zero elements should equal the number of manipulator DOF.
    //
    // For example when using a 5 DOF manipulator rotation about the wrist z-axis
    // might be unimportant in which case <literal>M = [1 1 1 1 1 0]</literal>.
    //
    // In all cases if T is 4x4xM it is taken as a homogeneous transform sequence 
    // and ikine() returns the joint coordinates corresponding to each of the 
    // transforms in the sequence.  Q is MxN where N is the number of robot joints.
    // The initial estimate of Q for each time step is taken as the solution 
    // from the previous time step.
    //
    //
// <refsection>
// <title>Notes</title>
    //  <itemizedlist>
    // <listitem><para>Solution is computed iteratively.</para></listitem>
    // <listitem><para>Solution is sensitive to choice of initial gain.  The variable
    //   step size logic (enabled by default) does its best to find a balance
    //   between speed of convergence and divergence.</para></listitem>
    // <listitem><para>Some experimentation might be required to find the right values of 
    //   tol, ilimit and alpha.</para></listitem>
    // <listitem><para>The pinv option sometimes leads to much faster convergence.</para></listitem>
    // <listitem><para>The tolerance is computed on the norm of the error between current
    //   and desired tool pose.  This norm is computed from distances
    //   and angles without any kind of weighting.</para></listitem>
    // <listitem><para>The inverse kinematic solution is generally not unique, and 
    //   depends on the initial guess Q0 (defaults to 0).</para></listitem>
    // <listitem><para>The default value of Q0 is zero which is a poor choice for most
    //   manipulators (eg. puma560, twolink) since it corresponds to a kinematic
    //   singularity.</para></listitem>
    // <listitem><para>Such a solution is completely general, though much less efficient 
    //   than specific inverse kinematic solutions derived symbolically, like
    //   ikine6s or ikine3.</para></listitem>
    // <listitem><para>This approach allows a solution to obtained at a singularity, but 
    //   the joint angles within the null space are arbitrarily assigned.</para></listitem>
    // <listitem><para>Joint offsets, if defined, are added to the inverse kinematics to 
    //   generate Q.</para></listitem>
    //  </itemizedlist>
    // </refsection>
    //
    // See also
    // fkine
    // tr2delta
    // jacob0
    // ikine6s
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    [%nargout, %nargin] = argn(0);
    //set default parameters for solution
    qt=[];
    opt.ilimit = 1000;
    opt.tol = 1e-6;
    opt.alpha = 1;
    opt.plot = %f;
    opt.pinv = %f;
    opt.varstep = %t;
    opt.verbose = 0;

    q_tmp = []
    dq_tmp = []
    e_tmp = []
    ne_tmp = []
    alpha_tmp = []
    if ~isempty(varargin) then
        for i = 1:(length(varargin)),
            if type(varargin(i))==10 then

                if ~strcmp(string(varargin(i)),'pinv') then
                    opt.pinv = ~opt.pinv;
                end
                if ~strcmp(string(varargin(i)),'novarstep') then
                    opt.varstep = ~opt.varstep;
                end
                if ~strcmp(string(varargin(i)),'plot') then
                    opt.plot = ~opt.plot;
                end
                if ~strcmp(string(varargin(i)),'verbose') then
                    opt.verbose = 1;
                end

                if varargin(i)=='ilimit'  then
                    opt.ilimit = varargin(i+1);
                elseif varargin(i) == 'tol'
                    opt.tol = varargin(i+1);
                elseif varargin(i) == 'alpha' then
                    opt.alpha = varargin(i+1);
                elseif varargin(i) == 'verbose=2' then
                    opt.verbose = 2;
                end
            end
        end 
    end

    n = robot.n;

    // ikine(robot,tr,q)
    if ~isempty(varargin)& isvector(varargin(1)) then
        q = varargin(1);
        if isempty(q) then
            q =zeros(1,n);
        else
            q = q(:)';
        end
    else 
        q=zeros(1,n);
    end

    // ikine(robot,tr,q,m)
    if  length(varargin)>1 & isvector(varargin(2)) then
        m = varargin(2);
        m = m(:);
        if length(m)~= 6 then
            error('Mask matrix should have 6 elements');
        end
        if length(find(m)) ~= n then 
            error('Mask matrix must have same number of 1s as robot DOF')
        end
    else
        if n<6 then
            error('For a manipulator with fewer than 6DOF a mask matrix argument must be specified');  
        end
        m = ones(6, 1);
    end


    //    % make this a logical array so we can index with it
    //    m = logical(m);

    npoints = size(tr,3);    // number of points
    qt = zeros(npoints, n);  // preallocate space for results
    tcount = 0;              // total iteration count

    if ~ishomog(tr) then
        error('ikine:badarg:    T is not a homog xform');
    end

    J0 = jacob0(robot, q);
    //J0 = J0(m, m);
    //disp(cond(J0))
    if cond(J0) > 100 then
        warning(msprintf('ikine:singular \nInitial joint angles results in near-singular configuration, this may slow convergence'));
    end

    history = [];
    failed = %f;
    for i=1:npoints
        T=tr(:,:,i);

        nm = %inf;
        eprev = %inf;
        saves.e = [%inf %inf %inf %inf %inf %inf];
        saves.q = [];
        count = 0;

        while %t,
            // update the count and test against iteration limit
            count = count +1;
            if count > opt.ilimit then
                warning(msprintf(gettext('ikine: iteration limit %d exceeded (row %d), final err %f'), opt.ilimit, i, nm));
                failed = %t;
                q = %nan *ones(1,n);
                break;
            end

            // compute the error
            e = tr2delta(fkine(robot,q'),T);

            if opt.varstep
                // test against last best error, only consider the DOF of interest
                // Changed from Scilab 5.4.1 to 5.5.0
                // isnan(norm(saves.e)) -> isinf(saves.e)
                if isinf(saves.e) then
                    norm_save_e = %inf;
                else
                    norm_save_e = norm(saves.e);
                end
                if isinf(e) then
                    norm_e = %inf;
                else
                    norm_e = norm(e);
                end
                //mprintf("norm(e)= %f \t norm(saves.e)= %f\n",norm_e,norm_save_e)
                if norm_e <= norm_save_e
                    // error reduced,
                    // let's saves current state of solution and rack up the step size
                    saves.q = q;
                    saves.e = e;
                    opt.alpha = opt.alpha * (2.0^(1.0/8));
                    if opt.verbose > 1
                        mprintf('raise alpha to %f\n', opt.alpha);
                    end
                else
                    // rats!  error got worse,
                    // restore to last good solution and reduce step size
                    q = saves.q;
                    e = saves.e;
                    opt.alpha = opt.alpha * 0.5;
                    if opt.verbose > 1
                        mprintf('drop alpha to %f\n', opt.alpha);
                    end
                end
            end
            // compute the Jacobian
            J = jacob0(robot, q);
            // compute change in joint angles to reduce the error, 
            // based on the square sub-Jacobian
            if opt.pinv then
                dq = opt.alpha * pinv(J) * e(:);
            else
                dq = J' * e(:);
                dq = opt.alpha * dq;
            end

            // diagnostic stuff
            if opt.verbose > 1
                mprintf('%d:%d: |e| = %f\n', i, count, nm);
                mprintf('       e  = '); disp(e');
                mprintf('       dq = '); disp(dq');
            end
            if opt.plot
                h.q = q';
                h.dq = dq;
                h.e = e;
                h.ne = nm;
                h.alpha = opt.alpha;
                q_tmp = [q_tmp;q'];
                dq_tmp = [dq_tmp,dq];
                e_tmp = [e_tmp,e];
                ne_tmp = [ne_tmp,nm];
                alpha_tmp = [alpha_tmp,opt.alpha];
            end

            // update the estimated solution
            q = q + dq';
            nm = norm(e);
            
            //Changed from Scilab 5.4.1 to 5.5.0
            //  norm(e) > 1.5*norm(eprev)
            if isinf(eprev) then
               norm_eprev = %nan;
            else
               norm_eprev = norm(eprev);
            end
            if norm(e) > 1.5*norm_eprev then
                warning(msprintf(gettext('ikine:diverged \nsolution diverging, try reducing alpha')));
            end
            eprev = e;

            if nm <= opt.tol
                break;
            end

        end // end ikine solution for tr(:,:,i)
        qt = q;//qt(i,:) = q';
        tcount = tcount + count;
        if opt.verbose then
            mprintf('%d iterations\n', count);
        end
    end
    if opt.verbose & npoints > 1 then
        mprintf('TOTAL %d iterations\n', tcount);
    end

    history.q = q_tmp;
    history.dq = dq_tmp;
    history.e = e_tmp;
    history.ne  = ne_tmp;
    history.alpha = alpha_tmp;
    // plot evolution of variables
    if opt.plot then
        scf(1);
        plot([history.q]');
        xlabel('iteration');
        ylabel('q');
        xgrid

        scf(2);
        plot([history.dq]');
        xlabel('iteration');
        ylabel('dq');
        xgrid

        scf(3);
        plot([history.e]');
        xlabel('iteration');
        ylabel('e');
        xgrid

        scf(4);
        plot(log([history.ne]));
        xlabel('iteration');
        ylabel('|e|');
        xgrid

        scf(5);
        plot([history.alpha]);
        xlabel('iteration');
        ylabel('\alpha');
        xgrid

        if %nargout > 1 then
            histout = history;
        end
    end
endfunction
