//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [x, y, typ] = ikine6s_if(job, arg1, arg2)
	// ikine6s_if
	//
	// Description
	// This block computes the inverse kinematics for a robot with spherical wrist in a certain configuration. 
	// A homogeneous transform for the location of the end-effector is received as input and joint coordinates is returned as output.
	//
	// This block has one regular input.
	//
	// This block has one regular output. It represent a set of joint coordinates and therefore the number of its elements must equal the number of robot's constituent links.
	//
	// You can specify as block parameter the robot model robot for which you want compute inverse kinematic.
	//
	// <refsection>
    // <title>Dialog Box</title>
    // <variablelist>
    //   <varlistentry><term>Robot object    :</term>
    // 	<listitem><para>SerialLink. It represents the robot model to be simulated. Note that this variable is a symbolic parameter which has to be defined in the context of the Xcos diagram in order to be used.</para></listitem></varlistentry>
    //   <varlistentry><term>Configuration String    :</term>
    // 	<listitem><para>Configuration string. Initial value: luf</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
	//
	// See also
	// ikine6s
	// ikine
	// ikine_if
	// 
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    x = [];
    y = [];
    typ = [];
    in_label = ['T']; 
    out_label = ['q'];
    style = 'blockWithLabel;fillColor=red;displayedLabel=ikine6s;fontStyle=1;fontSize=12';
    select job,
    case "set" then
        x = arg1;
        graphics = arg1.graphics;
        exprs = graphics.exprs;
        model = arg1.model;
        config = "";
        while %T do,
            [ok, robot_name, config, exprs] = getvalue(..
            "Robot inverse kinematics",..
            ["Robot object";..
            "Configuration String"],..
            list("str", -1, "str", -1),..
            exprs);
            robot = evstr(robot_name);
            if ok & typeof(robot) ~= "Robot" then
                // not a robot
                messagebox("invalid block parameter: required a robot object");
                ok = %F;
            elseif ok & robot.n == 0 then
                ok = %F;
            end
            if ~ok then
                break;
            end
            [model, graphics, ok] = set_io(model, graphics, list([4, 4], 1), list([robot.n, 1], 1) , [], []);
            if ok then
                model.opar = list(robot.links, robot.base, robot.tool, robot.offset, robot.mdh);        
                model.rpar = [config];
                graphics.exprs = exprs;
                style = 'blockWithLabel;fillColor=red;displayedLabel=ikine6s <BR><BR> %1$s;fontStyle=1;fontSize=12';
                graphics.style = style;
                x.graphics = graphics;
                x.model = model;
                break
            end
        end

    case "define" then
        model = scicos_model();
        model.sim = list("ikine6s_cf", 5);
        model.in = 4;
        model.in2 = 4;
        model.intyp = 1;
        model.out = -1;
        model.out2 = 1;
        model.outtyp = 1;
        model.opar = list();
        model.blocktype = "d";
        model.dep_ut = [%T %F];
        exprs = ["robot";"luf"];
        x = standard_define([4 4], model, exprs, "");
        x.graphics.style = style;
        x.graphics.out_label = out_label;
        x.graphics.in_label = in_label;

    end

endfunction
