//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [x, y, typ] = jtraj_if(job, arg1, arg2)
	// jtraj_if
	//
	// Description
	// This block generates quintic polynomial to move from intial (q0) to final (qf) joint angles as specified.
	// Initial and final velocities are assumed to be zero. Output joint angles as well as joint angle rates.
	//
	// The number of output of this block is three. The first output represents the joint state vector q.
	// The second output is the joint velocity vector qd. The third output represents the joint acceleration vector qdd.
	// Each of these vector valued signals must have n elements, where n is the number of elements of the vectors q0 and qf.
	//
	// <refsection>
    // <title>Dialog Box</title>
    // <variablelist>
    //   <varlistentry><term>q0     :</term>
    // 	<listitem><para>n-element vector. It describes the initial joint angles. Initial value: [0,0,0,0,0,0]</para></listitem></varlistentry>
    //   <varlistentry><term>qf    :</term>
    // 	<listitem><para>n-element vector. It describes the final joint angles. Initial value: [%pi/4,%pi/2,-%pi/2,0,0,0]</para></listitem></varlistentry>
    //   <varlistentry><term>tmax    :</term>
    // 	<listitem><para>scalar. The final time to complete the motion. Initial value: 10</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
	//
	// See also
	// jtraj
	//
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci
	
    style = 'blockWithLabel;fillColor=red;displayedLabel=jtraj;fontStyle=1;fontSize=12;align=left';
    x = [];
    y = [];
    typ = [];
    select job,
        case "set" then
            x = arg1;
            graphics = arg1.graphics;
            exprs = graphics.exprs;
            model = arg1.model;
            while %T do,
                [ok, q0, qf, tmax, exprs] = scicos_getvalue(..
                    ["Joint interpolated trajectory"],..
                    ["q0";..
                    "qf";..
                    "tmax"],..
                    list("vec", -1,"vec", -1,"vec", 1),..
                    exprs);
                if length(q0) ~= length(qf) then
                    message("q0 and qf must be same length");
                    ok = %F;
                elseif (length(q0) ~= 0) &..
                        ((size(q0,1) ~= 1 & size(q0,2) ~= 1) | (size(qf,1) ~= 1 & size(qf,2) ~= 1)) then
                    message("q0 and qf must be vectors");
                    ok = %F;
                end
                if ~ok then
                    break;
                end
                n = length(q0);
                [model, graphics, ok] = set_io(model, graphics, list(), list([n, 1; n, 1; n, 1], [1; 1; 1]), [], []);
                if ok then
                    t = [0:100].' / 100 * tmax;
                    [q, qd, qdd] = jtraj(q0, qf, t);
                    graphics.exprs = exprs;
                    model.rpar = [q, qd, qdd, t];
                    model.ipar=n;
                    graphics.out_label = ['q';'qd';'qdd'];
                    graphics.style =  style;
                    x.graphics=graphics;
                    x.model=model;
                    break;
                end
            end

        case "define" then
            q0 = [0, 0, 0, 0, 0, 0];
            qf = [%pi/4, %pi/2, -%pi/2, 0, 0, 0];
            tmax = 10;
            t = [0:100].' / 100 * tmax;
            [q, qd, qdd] = jtraj(q0, qf, t);
            model = scicos_model();
            model.sim = list("jtraj_cf",5);
            model.in = [];
            model.in2 = [];
            model.intyp = [];
            model.out = [6; 6; 6];
            model.out2 = [1; 1; 1];
            model.outtyp = [1; 1; 1];
            model.evtin = [];
            model.evtout = [];
            model.rpar = [q, qd, qdd, t];
            model.ipar = 6;
            model.blocktype = "d";
            model.dep_ut = [%T %T];
            exprs = [sci2exp(q0); sci2exp(qf); sci2exp(tmax)];
            x = standard_define([4 4], model, exprs, "");
            x.graphics.out_label = ['q';'qd';'qdd'];
            x.graphics.style = style;
    end

endfunction
