//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [s,sd,sdd] = lspb(q0, q1, t, V,opt)
    // Linear segment with parabolic blend
    //
    // Calling Sequence
    // [S,SD,SDD] = LSPB(S0, SF, M)
    // [S,SD,SDD] = LSPB(S0, SF, M, V)
    // [S,SD,SDD] = LSPB(S0, SF, T)
    // [S,SD,SDD] = LSPB(S0, SF, T, V)
    // [S,SD,SDD] = LSPB(S0, SF, M,'plot')
    // [S,SD,SDD] = LSPB(S0, SF, M, V,'plot')
    //
    // Description
    // <literal>[S,SD,SDD] = LSPB(S0, SF, M)</literal> is a scalar trajectory (Mx1) that varies 
    // smoothly from S0 to SF in M steps using a constant velocity segment and 
    // parabolic blends (a trapezoidal path).  Velocity and acceleration can be
    // optionally returned as SD (Mx1) and SDD (Mx1).
    //
    // <literal>[S,SD,SDD] = LSPB(S0, SF, M, V)</literal> as above but specifies the velocity of 
    // the linear segment which is normally computed automatically.
    //
    // <literal>[S,SD,SDD] = LSPB(S0, SF, T)</literal> as above but specifies the trajectory in 
    // terms of the length of the time vector T (Mx1).
    //
    // <literal>[S,SD,SDD] = LSPB(S0, SF, T, V)</literal> as above but specifies the velocity of 
    // the linear segment which is normally computed automatically and a time
    // vector.
    //
    // <refsection>
    // <title>Notes</title>
    //  <itemizedlist>
    // <listitem><para>If no output arguments are specified S, SD, and SDD are plotted.</para></listitem>
    // <listitem><para>For some values of V no solution is possible and an error is flagged.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // tpoly 
    // jtraj
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    [%nargout, %nargin] = argn(0);    

    t0 = t;
    if sum(length(t))==1 then //isscalar(t)
        t = (0:t-1)';
    else
        t = t(:);
    end

    tf = max(t(:));

    if %nargin < 4 then
        // if velocity not specified, compute it
        V = (q1-q0)/tf * 1.5;
    elseif %nargin == 4
        if V == 'plot' then
            opt = 'plot'
            V = (q1-q0)/tf * 1.5;
        else
            if V < (q1-q0)/tf then
                error('V too small');
            elseif V > 2*(q1-q0)/tf
                error('V too big');
            end
            opt = "";
        end
    end

    if q0 == q1 then
        s = ones(size(t)) * q0;
        sd = zeros(size(t));
        sdd = zeros(size(t));
        return
    end

    tb = (q0 - q1 + V*tf)/V;
    a = V/tb;

    p = zeros(length(t), 1);
    pd = p;
    pdd = p;

    for i = 1:length(t)
        tt = t(i);

        if tt <= tb then
            // initial blend
            p(i) = q0 + a/2*tt^2;
            pd(i) = a*tt;
            pdd(i) = a;
        elseif tt <= (tf-tb)
            // linear motion
            p(i) = (q1+q0-V*tf)/2 + V*tt;
            pd(i) = V;
            pdd(i) = 0;
        else
            // final blend
            p(i) = q1 - a/2*tf^2 + a*tf*tt - a/2*tt^2;
            pd(i) = a*tf - a*tt;
            pdd(i) = -a;
        end
    end

    select %nargout
    case 1
        s = p;
    case 2
        s = p;
        sd = pd;
    case 3
        s = p;
        sd = pd;
        sdd = pdd;
    end
    if (%nargin>= 4 & opt == 'plot') then
        if sum(length(t0))==1 then//isscalar
            // for scalar time steps, axis is labeled 1 .. M
            xt = t+1;
        else
            // for vector time steps, axis is labeled by vector M
            xt = t;
        end

        clf
        subplot(311)
        // highlight the accel, coast, decel phases with different
        // colored markers
        drawlater()
        k = t<= tb;
        plot(xt(k), p(k), 'r-o');
        k = (t>=tb) & (t<= (tf-tb));
        plot(xt(k), p(k), 'b-o');
        k = t>= (tf-tb);
        plot(xt(k), p(k), 'g-o');
        ylabel('s');
        drawnow()

        subplot(312)
        plot(xt, pd);  ylabel('sd');

        subplot(313)
        plot(xt, pdd); ylabel('sdd');
        if sum(length(t0))<>1 then
            xlabel('time')
            //        else
            //            for c=get(gcf, 'Children');
            //                set(c, 'XLim', [1 t0]);
            //            end
        end
    end
endfunction
