//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function mdl_puma560()
    // Create model of Puma 560 manipulator
    //
    // Calling Sequence
    // mdl_puma560
    //
    // Description
    // Script creates the workspace variable p560 which describes the 
    // kinematic and dynamic characteristics of a Unimation Puma 560 manipulator
    // using standard DH conventions.
    // The model includes armature inertia and gear ratios.
    //
    // Also define the workspace vectors:
    //    
    //   qz         zero joint angle configuration
    //    
    //   qready         vertical 'READY' configuration
    //
    //   qstretch   arm is stretched out in the X direction
    //    
    //   qn         arm is at a nominal non-singular configuration
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>the value of m1 is given as 0 here.  Armstrong found no value for it
    // and it does not appear in the equation for tau1 after the substituion
    // is made to inertia about link frame rather than COG frame.
    // updated:
    //    
    // 2/8/95  changed D3 to 150.05mm which is closer to data from Lee, AKB86 and Tarn
    //  fixed errors in COG for links 2 and 3
    //    
    // 29/1/91 to agree with data from Armstrong etal.  Due to their use
    //  of modified D&H params, some of the offsets Ai, Di are
    //  offset, and for links 3-5 swap Y and Z axes.
    //    
    // 14/2/91 to use Paul's value of link twist (alpha) to be consistant
    //  with ARCL.  This is the -ve of Lee's values, which means the
    //  zero angle position is a righty for Paul, and lefty for Lee.
    //  Note that gravity load torque is the motor torque necessary
    //  to keep the joint static, and is thus -ve of the gravity
    //  caused torque.
    //
    // 8/95 fix bugs in COG data for Puma 560. This led to signficant errors in
    //  inertia of joint 1.  </para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // Bibliography
    // [1] "A search for consensus among model parameters reported for the PUMA 560 robot", P. Corke and B. Armstrong-Helouvry, Proc. IEEE Int. Conf. Robotics and Automation, (San Diego), pp. 1608-1613, May 1994.
    // [2] "A combined optimization method for solving the inverse kinematics problem...", Wang & Chen, IEEE Trans. RA 7(4) 1991 pp 489-
    //
    // See also 
    // SerialLink
    // mdl_puma560akb 
    // mdl_stanford
    // mdl_twolink
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    L = list();
    deg = %pi/180;

    ////          a       alpha           d        theta
    ////l1 = Link(0,%pi/2,0,0,0,"std");
    ////l2 = Link(0,%pi/2,0,0,0,"std");
    ////l1 = Link( 0      ,  %pi/2     ,  0      ,  0     );
    //l2 = Link(0.4318  ,  0         ,  0      ,  0     );
    //l3 = Link(0.0203  ,   -%pi/2   , 0.15005 ,  0     );
    //l4 = Link(0       ,  %pi/2     ,  0.4318 ,  0     );
    //l5 = Link(0       ,  -%pi/2    ,  0      ,  0     );
    //l6 = Link(0       ,  0         ,  0      ,  0     );
    //
    //l1.I = [0, 0.35, 0, 0, 0, 0];
    //l1.r = [0, 0, 0];
    //l1.m = 0;
    //l1.Jm = 200e-6;
    //l1.G = -62.6111;
    //l1.B = 1.48e-3;
    //l1.Tc = [0.395 -0.435];
    ////'qlim', [-160 160]*deg
    //l2.I = [0.13, 0.524, 0.539, 0, 0, 0];
    //l2.r = [-0.3638, 0.006, 0.2275];
    //l2.m = 17.4;
    //l2.Jm = 200e-6;
    //l2.G = 107.815;
    //l2.B = .817e-3;
    //l2.Tc = [0.126 -0.071];
    ////    'qlim', [-45 225]*deg );
    //
    //l3.I = [0.066, 0.086, 0.0125, 0, 0, 0];
    //l3.r = [-0.0203, -0.0141, 0.070];
    //l3.m = 4.8;
    //l3.Jm = 200e-6;
    //l3.G = -53.7063;
    //l3.B = 1.38e-3;
    //l3.Tc = [0.132, -0.105];
    ////    'qlim', [-225 45]*deg );
    //l4.I = [1.8e-3, 1.3e-3, 1.8e-3, 0, 0, 0];
    //l4.r = [0, 0.019, 0];
    //l4.m = 0.82;
    //l4.Jm = 33e-6;
    //l4.G = 76.0364;
    //l4.B = 71.2e-6;
    //l4.Tc = [11.2e-3, -16.9e-3];
    ////    'qlim', [-110 170]*deg);
    ////
    //l5.I = [0.3e-3, 0.4e-3, 0.3e-3, 0, 0, 0];
    //l5.r = [0, 0, 0];
    //l5.m = 0.34;
    //l5.Jm = 33e-6;
    //l5.G = 71.923;
    //l5.B = 82.6e-6;
    //l5.Tc = [9.26e-3, -14.5e-3];
    ////    'qlim', [-100 100]*deg );
    //l6.I = [0.15e-3, 0.15e-3, 0.04e-3, 0, 0, 0];
    //l6.r = [0, 0, 0.032];
    //l6.m = 0.09;
    //l6.Jm = 33e-6;
    //l6.G = 76.686;
    //l6.B = 36.7e-6;
    //l6.Tc = [3.96e-3, -10.5e-3];
    ////    'qlim', [-266 266]*deg );
    //
    L(1) = Link('d', 0, 'a', 0, 'alpha', %pi/2, ...
    'I', [0, 0.35, 0, 0, 0, 0], ...
    'r', [0, 0, 0], ...
    'm', 0, ...
    'Jm', 200e-6, ...
    'G', -62.6111, ...
    'B', 1.48e-3, ...
    'Tc', [0.395 -0.435], ...
    'qlim', [-160 160]*deg );

    L(2) = Link('d', 0, 'a', 0.4318, 'alpha', 0, ...
    'I', [0.13, 0.524, 0.539, 0, 0, 0], ...
    'r', [-0.3638, 0.006, 0.2275], ...
    'm', 17.4, ...
    'Jm', 200e-6, ...
    'G', 107.815, ...
    'B', .817e-3, ...
    'Tc', [0.126 -0.071], ...
    'qlim', [-45 225]*deg );

    L(3) = Link('d', 0.15005, 'a', 0.0203, 'alpha', -%pi/2,  ...
    'I', [0.066, 0.086, 0.0125, 0, 0, 0], ...
    'r', [-0.0203, -0.0141, 0.070], ...
    'm', 4.8, ...
    'Jm', 200e-6, ...
    'G', -53.7063, ...
    'B', 1.38e-3, ...
    'Tc', [0.132, -0.105], ...
    'qlim', [-225 45]*deg );

    L(4) = Link('d', 0.4318, 'a', 0, 'alpha', %pi/2,  ...
    'I', [1.8e-3, 1.3e-3, 1.8e-3, 0, 0, 0], ...
    'r', [0, 0.019, 0], ...
    'm', 0.82, ...
    'Jm', 33e-6, ...
    'G', 76.0364, ...
    'B', 71.2e-6, ...
    'Tc', [11.2e-3, -16.9e-3], ...
    'qlim', [-110 170]*deg);

    L(5) = Link('d', 0, 'a', 0, 'alpha', -%pi/2,  ...
    'I', [0.3e-3, 0.4e-3, 0.3e-3, 0, 0, 0], ...
    'r', [0, 0, 0], ...
    'm', 0.34, ...
    'Jm', 33e-6, ...
    'G', 71.923, ...
    'B', 82.6e-6, ...
    'Tc', [9.26e-3, -14.5e-3], ...
    'qlim', [-100 100]*deg );


    L(6) = Link('d', 0, 'a', 0, 'alpha', 0,  ...
    'I', [0.15e-3, 0.15e-3, 0.04e-3, 0, 0, 0], ...
    'r', [0, 0, 0.032], ...
    'm', 0.09, ...
    'Jm', 33e-6, ...
    'G', 76.686, ...
    'B', 36.7e-6, ...
    'Tc', [3.96e-3, -10.5e-3], ...
    'qlim', [-266 266]*deg );

    //L = list(l1,l2,l3,l4,l5,l6);
    p560 = SerialLink(L, 'name', 'Puma 560', ...
    'manuf', 'Unimation', 'comment', 'viscous friction; params of 8/95');



    //
    // some useful poses
    //
    qz = [0 0 0 0 0 0]; // zero angles, L shaped pose
    qready = [0 %pi/2 -%pi/2 0 0 0]; // ready pose, arm up
    qstretch = [0 0 -%pi/2 0 0 0];
    qn=[0 %pi/4 %pi 0 %pi/4  0];


    [p560,qz,qready,qstretch,qn]=resume(p560,qz,qready,qstretch,qn)
endfunction
