//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function mdl_twolink()
    // Create model of a simple 2-link mechanism
    //
    // Calling Sequence
    // mdl_twolink
    //
    // Description
    // Script creates the workspace variable tl which describes the 
    // kinematic and dynamic characteristics of a simple planar 2-link mechanism.
    //
    // Also defines the vector:
    //
    //   qz   corresponds to the zero joint angle configuration.
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>It is a planar mechanism operating in the XY (horizontal) plane and is 
    //   therefore not affected by gravity.</para></listitem>
    // <listitem><para>Assume unit length links with all mass (unity) concentrated at the joints.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // Bibliography
    //  [1] Based on Fig 3-6 (p73) of Spong and Vidyasagar (1st edition).  
    //
    // See also 
    // SerialLink
    // mdl_puma560 
    // mdl_stanford
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci
    
    a1 = 1;
    a2 = 1;
    L = list();
    //   theta d a alpha 
    L(1) = Link( 0  ,  0 ,  a1 , 0, 'standard');
    L(2) = Link( 0  ,  0 ,  a2 , 0, 'standard');

    L(1).m = 1;
    L(1).r = [-0.5 0 0];
    L(1).I = zeros(3,3);
    L(1).G = 0;
    L(1).Jm = 0;
    L(1).B = 0;
    L(2).m = 1;
    L(2).r = [-0.5 0 0];
    L(2).I = zeros(3,3);
    L(2).G = 0;
    L(2).Jm = 0;
    L(2).B = 0;
    twolink = SerialLink(L, "name", "two link", "comment", "from Spong, Hutchinson, Vidyasagar");
    qz = [0 0];
    qn = [%pi/6, -%pi/6];

    [twolink,qz,qn]=resume(twolink,qz,qn);

endfunction
