function [S,Sd,Sdd] = mtraj(tfunc, q0, qf, M,opt)
    // Multi-axis trajectory between two points
    //
    // Calling Sequence
    // [Q,QD,QDD] = MTRAJ(TFUNC, Q0, QF, M)
    // [Q,QD,QDD] = MTRAJ(TFUNC, Q0, QF, M,'plot')
    //
    // Description
    // <literal>[Q,QD,QDD] = MTRAJ(TFUNC, Q0, QF, M)</literal> is a multi-axis trajectory (MxN) varying
    // from state Q0 (1xN) to QF (1xN) according to the scalar trajectory function 
    // TFUNC in M steps. Joint velocity and acceleration can be optionally returned as 
    // QD (MxN) and QDD (MxN) respectively.  The trajectory outputs have one row per 
    // time step, and one column per axis.
    //
    // The shape of the trajectory is given by the scalar trajectory function TFUNC.
    // Possible values of TFUNC are 'lspb' for a trapezoidal trajectory, or
    // 'tpoly' for a polynomial trajectory.
    //
    // <literal>[Q,QD,QDD] = MTRAJ(TFUNC, Q0, QF, T)</literal> as above but specifies the trajectory 
    // length in terms of the length of the time vector T (Mx1).
    //
    // <refsection>
    // <title>Notes</title>
    //  <itemizedlist>
    // <listitem><para>If no output arguments are specified Q, QD, and QDD are plotted.</para></listitem>
    // <listitem><para> When TFUNC is @tpoly the result is functionally equivalent to JTRAJ except 
    //   that no initial velocities can be specified. JTRAJ is computationally a little
    //   more efficient.</para></listitem>
    // </itemizedlist>
	// </refsection>
    //
    // See also 
    // jtraj
    // mstraj
    // lspb
    // tpoly
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    //
    // This file is part of The Robotics Toolbox for Matlab (RTB).

    [%nargout,%nargin] = argn(0)
    //    if type(tfunc)<>13; //~isa(tfunc, 'function_handle')
    //        error('first argument must be a function handle');
    //    end
    if tfunc <> 'tpoly'  & tfunc <> 'lspb' then
        error('first argument must be one of these string: ""tpoly"",""lspb""");
    end

    M0 = M;
    if sum(length(M))<>1 then//~isscalar(M)
        M = length(M);
    end
    if numcols(q0) ~= numcols(qf)
        error('must be same number of columns in q0 and qf')
    end

    //    s = zeros(M, numcols(q0));
    //    sd = zeros(M, numcols(q0));
    //    sdd = zeros(M, numcols(q0));

    s = [];sd = [];sdd = [];
    for i=1:numcols(q0)
        // for each axis
        execstr("[ts,tsd,tsdd] = "+tfunc+"(q0(i), qf(i), M)");
        s = [s,ts]; sd = [sd,tsd];  sdd = [sdd,tsdd];
    end

    // - If no output arguments are specified S, SD, and SDD are plotted 
    //   against time.


    select %nargout
    case 1
        S = s;
    case 2
        S = s;
        Sd = sd;
    case 3
        S = s;
        Sd = sd;
        Sdd = sdd;
    end
    
    if (%nargin == 5 & opt == 'plot') then
        clf

        if sum(length(M0))==1 then//isscalar
            t = [1:M0]';
        else
            t = M0;
        end
        subplot(311)
        plot(t, s); ylabel('s');

        subplot(312)
        plot(t, sd); ylabel('sd');

        subplot(313)
        plot(t, sdd); ylabel('sdd');
        if sum(length(M0))<>1//~isscalar(M0)
            xlabel('time')
//        else
//            for c=get(gcf, 'Children');
//                set(c, 'XLim', [1 M0]);
//            end
        end
    end
endfunction
