//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function  r2 = nofriction(r, varargin)
    // Remove friction 
    //
    // Calling Sequence
    // RNF = nofriction(ROBOT)
    // RNF = nofriction(ROBOT, OPT)
    // LN = nofriction(L)
    // LN = nofriction(L,OPT)
    //
    // Parameters
    // L    : Link object
    // ROBOT : SerialLink object
    // OPT	: valid string options "all", "viscous", "coulomb"
    //
    // Description
    // <literal>RNF = nofriction(ROBOT)</literal> is a robot object with the same 
    //  parameters as ROBOT but with non-linear (Coulomb) friction 
    //  coefficients set to zero. 
    //
    // <literal>RNF = nofriction(ROBOT,'all')</literal> as above but all friction 
    //  coefficients set to zero.
    //
    // <literal>RNF = nofriction(ROBOT,'viscous')</literal> as above but only 
    //viscous friction coefficients are set to zero.
    //
    // <literal>LN = nofriction(L)</literal> is a link object with the same 
    //  parameters as L except nonlinear (Coulomb) friction parameter is zero.
    //
    // <literal>LN = nofriction(L,'all')</literal> as above except that viscous 
    // and Coulomb friction are set to zero.
    // 
    // <literal>LN = nofriction(L,'coulomb')</literal> as above except that Coulomb 
    // friction is set to zero.
    //
    // <literal>LN = nofriction(L,'viscous')</literal> as above except that 
    // viscous friction is set to zero.
    //
    // <refsection>
	// <title>Notes</title>
    //	<itemizedlist>
    // 	<listitem><para>Non-linear (Coulomb) friction can cause numerical problems when integrating
    //   the equations of motion (fdyn).</para></listitem>
    // 	<listitem><para>The resulting robot object has its name string prefixed with 'NF/'.</para></listitem>
    //  <listitem><para>Forward dynamic simulation can be very slow with finite Coulomb friction.</para></listitem>
    //	</itemizedlist>
	// </refsection>
    //
    // See also
    //	Link
    //	SerialLink
    //	fdyn
    //  friction.
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    if  typeof(r) == "Link" then

        r2 = Link(r);

        if isempty(varargin) then
            only = 'coulomb';
        else 
            only = varargin(1);
        end

        select only
        case 'all'
            r2(14) = 0;         //l2.B
            r2(15) = [0 0];     //l2.Tc
        case 'viscous'
            r2(14) = 0;         //l2.B 
        case 'coulomb'
            r2(15) = [0 0];     //12.Tc
        end

    elseif typeof(r)== "Robot" then
        r2 = SerialLink(r); // make a copy

        for j=1:r2.n
            r2.links(j) = nofriction(r.links(j),varargin());
        end
        r2.name = 'NF/'+r.name;

    else
        error(mprintf("Unknown type.\nUsage: nofriction(link,varargin) or nofriction(robot,varargin)\n varargin = {all,viscous,coulomb})"));
    end

endfunction


