//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [x, y, typ] = plot_if(job, arg1, arg2)
	// plot_if
	//
	// Description
	// This block creates a figure for displaying a graphical animation of a robot based on the kinematic model.
	// Initial and final velocities are assumed to be zero. Output joint angles as well as joint angle rates.
	//
	// This block has one regular input, the joint configuration vector.
	// This vector must have n elements, where n is the number of SerialLink's constituent links.
	//
	// This block works properly using the block SampleCLK instead of CLOCK_c. 
	// Through the clock's block is possible to determine the refresh time of the figure. 
	//
	// <refsection>
    // <title>Dialog Box</title>
    // <variablelist>
    //   <varlistentry><term>Robot object    :</term>
    // 	<listitem><para>SerialLink. It represents the robot model to be simulated. Note that this variable is a symbolic parameter which has to be defined in the context of the Xcos diagram in order to be used.</para></listitem></varlistentry>
    //   <varlistentry><term>Hold (yes/no)   :</term>
    // 	<listitem><para>string (yes or no). It can be used for drawing another SerialLink object in a existing figure. Initial value: no.</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
	//
	// See also
	// plot_robot
	// robot_if
	//
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci
    x = [];
    y = [];
    typ = [];
    in_label = ['q']; 
    style = 'blockWithLabel;fillColor=red;displayedLabel=Plot;fontStyle=1;fontSize=12;align=right';
    
    select job,
        case "set" then
            x = arg1;
            graphics = arg1.graphics;
            exprs = graphics.exprs;
            model = arg1.model;
            hold ="";
            while %T do,
                [ok, robot_name,hold, exprs] = scicos_getvalue(..
                    "Robot plot",..
                    ["Robot object";..
                     "Hold (yes/no)"],..
                    list("str", -1,"str",-1),..
                    exprs)
				robot = evstr(robot_name)
                if ok & typeof(robot) ~= "Robot" then
                    // not a robot
                    messagebox("invalid block parameter: required a robot object");
                    ok = %F;
                elseif ok & robot.n == 0 then
                    // a robot model to be simulated still must be specified
                    ok = %F;
                end
                if ~ok then
                    break;
                end
                [model, graphics, ok] = set_io(model, graphics, list([robot.n, 1], 1), list(), [], []);
                if ok then
                    model.opar = list(robot.links, robot.base, robot.tool, robot.offset, robot.name,hold);
                    model.evtin = 1;
                    model.blocktype = "d";
                    model.dep_ut = [%t %f];
                    model.odstate = list();
                    graphics.exprs = exprs;
                    graphics.in_label = in_label;
                    graphics.style ='blockWithLabel;fillColor=red;displayedLabel=Plot <BR><BR> %1$s;fontStyle=1;fontSize=12;align=right';
                    graphics.id = robot.name;
                    x.graphics = graphics;
                    x.model = model;
                    break
                end
            end

        case "define" then
            model = scicos_model();
            model.sim = list("plot_cf", 5);
            model.in = -1;
            model.in2 = 1;
            model.intyp = 1;
            model.opar = list();
            model.evtin = 1;
            model.blocktype = "d";
            model.dep_ut = [%t %f];
            exprs = ["robot","no"];
            x = standard_define([4 3], model, exprs, "");
            x.graphics.style = style;
            x.graphics.in_label = in_label;
    end

endfunction
