//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [o] = plot_options(robot,optin)
    // Returns an options structure
    //
    // Calling Sequence
    //  o = plot_options(robot, options)
    //
    // Description
    // Returns an options structure
    //
    // <refsection>
    //   <title>Options</title>
    //   <variablelist>
    //   <varlistentry><term>"workspace", W :</term>
    //         <listitem><para>size of robot 3D workspace, W = [xmn, xmx ymn ymx zmn zmx]</para></listitem></varlistentry>
    //   <varlistentry><term>"delay", d :</term>
    //         <listitem><para>delay betwen frames for animation (s)</para></listitem></varlistentry>
    //   <varlistentry><term>"fps",fps   :</term>
    //         <listitem><para>set number of frames per second for display</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]loop"  :</term>
    //         <listitem><para>loop over the trajectory forever</para></listitem></varlistentry>
    //   <varlistentry><term>"mag", scale :</term>
    //         <listitem><para>annotation scale factor</para></listitem></varlistentry>
    //   <varlistentry><term>"cylinder", C :</term>
    //         <listitem><para>color for joint cylinders, C=[r g b]</para></listitem></varlistentry>
    //   <varlistentry><term>"ortho" :</term>
    //         <listitem><para> orthogonal camera view (default)</para></listitem></varlistentry>
    //   <varlistentry><term>"perspective"  :</term>
    //         <listitem><para>perspective camera view</para></listitem></varlistentry>
    //   <varlistentry><term>"xyz" :</term>
    //         <listitem><para> wrist axis label is XYZ</para></listitem></varlistentry>
    //   <varlistentry><term>"noa"   :</term>
    //         <listitem><para>wrist axis label is NOA</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]raise"  :</term>
    //         <listitem><para>autoraise the figure (very slow)</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]render"   :</term>
    //         <listitem><para>controls shaded rendering after drawing</para></listitem></varlistentry>
    //   <varlistentry><term> "[no]base" :</term>
    //         <listitem><para>controls display of base "pedestal"</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]wrist"  :</term>
    //         <listitem><para>controls display of wrist</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]shadow" :</term>
    //         <listitem><para>controls display of shadow</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]name"  :</term>
    //         <listitem><para>display the robot"s name</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]jaxes" :</term>
    //         <listitem><para> control display of joint axes</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]joints"   :</term>
    //         <listitem><para>controls display of joints</para></listitem></varlistentry>
    //   <varlistentry><term>"[no]hold"   :</term>
    //         <listitem><para>allows to draw another robot in the last figure created</para></listitem></varlistentry>
    //   <varlistentry><term>"movie"   :</term>
    //         <listitem><para>create a folder with png(default) images </para></listitem></varlistentry>
    //   <varlistentry><term>"movie","eps"   :</term>
    //         <listitem><para>create a folder with the eps images, supported formats are "bmp","gif","jpg","ppm","eps","pdf","svg","ps","emf" </para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    // process options
    o.erasemode = 'normal';
    o.joints = %t;
    o.wrist = %t;
    o.loop = %f;
    o.shadow = %t;
    o.wrist = %t;
    o.jaxes = %t;
    o.base = %t;
    o.wristlabel = 'xyz';
    o.perspective = %t;
    o.magscale = 1;
    o.name = %t;
    o.delay = 0;              
    o.fps = [];                 //unused. Use sampCLOCK block with "plot_it". 
    o.raise = %t;               //unused
    o.cylinder = [0 0 0.9];     //color!
    o.workspace = [];
    o.movie = [];
    o.render = %t;
    o.ortho = %t;
    o.perspective = %f;
    o.hold = %f;
    o.movie = %f;
    o.extens = "png";

    // read options string in the order
    //  1. robot.plotopt
    //  2. the SCI-file plotbotopt if it exists
    //  3. command line arguments
    [file_parameters,err] = fileinfo("plotbotopt.sci");
    if err == 0 then
        getf("plotbotopt.sci");
        options = rt_plotbotopt();
        options = lstcat(options, robot.plotopt, optin);
    else
        options = lstcat(robot.plotopt, optin);
    end
    i = 1;
    while i <= length(options),
        select convstr(options(i), "l")
        case "workspace" then
            o.workspace = options(i+1)
            i = i+1;
        case "mag" then
            o.magscale = options(i+1)
            i = i+1;
        case "fps" then
            o.fps = options(i+1)
            i = i+1;
        case "delay" then
            o.delay = options(i+1)
            i = i+1;
       case "cylinder" then
            o.cylinder = options(i+1)
            i = i+1;
        case "perspective" then
            o.projection = "perspective";
        case "ortho" then
            o.projection = "orthographic";
        case "erase" then
            o.erasemode = "copy";
        case "noerase" then
            o.erasemode = "none";
        case "base" then
            o.base = 1;
        case "nobase" then
            o.base = 0;
        case "loop" then
            o.repeat = options(i+1);
            i = i+1;
        case "noloop" then
            o.repeat = 1;
        case "name" then
            o.name = 1;
        case "noname" then
            o.name = 0;
        case "loop" then
            o.loop = 1;
        case "noloop" then
            o.loop = 0;
        case "wrist" then
            o.wrist = 1;
        case "nowrist" then
            o.wrist = 0;
        case "shadow" then
            o.shadow = 1;
        case "noshadow" then
            o.shadow = 0;
        case "xyz" then
            o.wristlabel = "XYZ";
        case "noa" then
            o.wristlabel = "NOA";
        case "joints" then
            o.joints = 1;
        case "nojoints" then
            o.joints = 0;
        case "jaxes" then
            o.jaxes = 1;
        case "nojaxes" then
            o.jaxes = 0;        
        case "hold" then
            o.hold = %t;
        case "nohold" then
            o.hold = %f;
        case "movie" then
            o.movie = %t;
            image_ext = ["bmp","gif","jpg","ppm","eps","pdf","svg","ps","emf"];
            if i+1 <= length(options) then
                for j=1:size(image_ext,2),
                    if options(i+1) == image_ext(j) then
                        o.extens = image_ext(j);
                        j = size(image_ext,2); //end
                        i = i+1;
                        break;
                    end
                end
            end
        else
            printf("unknown option: %s\n", string(options(i)));
            abort;
        end
        i = i+1;
    end

    if isempty(o.workspace) then
        //
        // simple heuristic to figure the maximum reach of the robot
        //
        L = robot.links;
        reach = 0;
        for i=1:robot.n
            reach = reach + abs(L(i).a) + abs(L(i).d);
        end
        reach=reach;
        o.workspace = [-reach reach -reach reach -reach reach];
        o.mag = reach/5;    //Robotic Toolbox:  o.mag = reach/10;
    else
        reach = min(abs(o.workspace));
    end
    o.mag = o.magscale * reach/5;   //Robotic Toolbox:  o.mag = reach/10;

    //    if ~isempty(o.fps) & o.fps > 0 then
    //        o.delay = 1/o.fps*1000000;
    //    end

endfunction
