//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function q = qscale(Q, r)
    // Interpolate rotations expressed by quaternion objects
    //
    // Calling Sequence
    // QI = qscale(Q,S)
    //
    // Description
    // <literal>QI = qscale(Q,S)</literal> is a unit-quaternion that interpolates between identity for S=0
    // to Q for S=1.  This is a spherical linear interpolation (slerp) that can
    // be interpretted as interpolation along a great circle arc on a sphere.
    //
    // If S is a vector QI is a list of quaternions, each element
    // corresponding to sequential elements of S.
    //
    // <refsection>
	// <title>Notes</title>
	// <itemizedlist>
    // <listitem><para>This is a spherical linear interpolation (slerp) that can be interpretted 
    //   as interpolation along a great circle arc on a sphere.</para></listitem>
    // </itemizedlist>
	// </refsection>
    //
    // See also 
    // Quaternion
    // ctraj
    // qinterp
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    q2 = qdouble(Q);

    if or(r<0) | (r>1)
        error('r out of range');
    end
    q1 = [1 0 0 0];         // identity quaternion
    theta = acos(q1*q2');

    if length(r) == 1
        if theta == 0
            q = Q;
        else
            q = Quaternion( (sin((1-r)*theta) * q1 + sin(r*theta) * q2) / sin(theta) )
            q=q.unit;
        end
    else
        q = list();
        R=r(:)'
        for count = 1:length(R)
            if theta == 0
                qq = Q;
            else
                qq = Quaternion( (sin((1-R(count))*theta) * q1 + sin(R(count)*theta) * q2) / sin(theta) );
                qq = qq.unit;
            end
            q(count) = qq;
            count = count + 1;
        end
    end
endfunction
