//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function T = r2t(R)
    // Convert rotation matrix to a homogeneous transform
    //
    // Description
    // T = r2t(R) is a homogeneous transform equivalent to an orthonormal 
    // rotation matrix R with a zero translational component.
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para> Works for T in either SE(2) or SE(3)</para></listitem>
    // <listitem><para> if R is 2x2 then T is 3x3, or </para></listitem>
    // <listitem><para> if R is 3x3 then T is 4x4.  </para></listitem>
    // <listitem><para> Translational component is zero. </para></listitem>
    // <listitem><para> For a rotation matrix sequence returns a homogeneous 
    // transform sequence.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // t2r
    //    
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    // check dimensions: R is SO(2) or SO(3)
    d = size(R);
    if d(1) ~= d(2)
        error('matrix must be square');
    end
    if ~or(d(1) == [2 3])
        error('argument is not a rotation matrix (sequence)');
    end

    Z = zeros(d(1),1);
    B = [Z' 1];

    if length(d) == 2
        // single matrix case
        T = [R Z; B];
    else
        //  matrix sequence case
        T = zeros(4,4,d(3));
        for i=1:d(3)
            T(:,:,i) = [R(:,:,i) Z; B];
        end
    end
endfunction
