//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function tau = rne(robot, varargin)
// Inverse dynamics
//
// Calling Sequence
// TAU = rne(R,Q, QD, QDD)
// TAU = rne(R,Q, QD, QDD, GRAV)
// TAU = rne(R,Q, QD, QDD, GRAV, FEXT)
// TAU = rne(R,X)
// TAU = rne(R,X, GRAV)
// TAU = rne(R,X, GRAV, FEXT)
// [TAU,WBASE] = rne(R,X, GRAV, FEXT)
//
// Description
// TAU = rne(R,Q, QD, QDD) is the joint torque required for the robot R
// to achieve the specified joint position Q, velocity QD and acceleration QDD.
//
// TAU = rne(R,Q, QD, QDD, GRAV) as above but overriding the gravitational 
// acceleration vector in the robot object R.
//
// TAU = rne(R,Q, QD, QDD, GRAV, FEXT) as above but specifying a wrench 
// acting on the end of the manipulator which is a 6-vector [Fx Fy Fz Mx My Mz].
//
// TAU = rne(R,X) as above where X=[Q,QD,QDD].
//
// TAU = rne(R,X, GRAV) as above but overriding the gravitational 
// acceleration vector in the robot object R.
// TAU = rne(R,X, GRAV, FEXT) as above but specifying a wrench 
// acting on the end of the manipulator which is a 6-vector [Fx Fy Fz Mx My Mz].
//
// [TAU,WBASE] = rne(R,X, GRAV, FEXT) as above but the extra output is the
// wrench on the base.
//
// If Q,QD and QDD (MxN), or X (Mx3N) are matrices with M rows representing a 
// trajectory then TAU (MxN) is a matrix with rows corresponding to each trajectory 
// step.
//
// <refsection>
// <title>Notes</title>
//  <itemizedlist>
//  <listitem><para>The robot base transform is ignored.</para></listitem>
//  <listitem><para>The torque computed contains a contribution due to armature
//   inertia and joint friction.</para></listitem>
//  <listitem><para>RNE can be either an M-file or a MEX-file.</para></listitem>
//  <listitem><para>See the README file in the mex folder for details on how to configure 
//   MEX-file operation.</para></listitem>
//  <listitem><para>The M-file is a wrapper which calls either RNE_DH or RNE_MDH depending on 
//   the kinematic conventions used by the robot object.</para></listitem>
//  <listitem><para>Currently the MEX-file version does not compute WBASE.</para></listitem>
// </itemizedlist>
// </refsection>
//
// See also 
// SerialLink
// accel
// gravload
// inertia
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014 - Davide Cappucci

    try
        tau = frne(robot, varargin(:));
    catch
        warning(msprintf(gettext("%s: Frne not found.\n"),"rne"));
        if robot.mdh == 0 then
            tau = rne_dh(robot, varargin(:));
        else
            tau = rne_mdh(robot, varargin(:));
        end
    end

endfunction
