//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [x, y, typ] = robot_if(job, arg1, arg2)
	// robot_if
	//
	// Description
	// This block is a Super Block and it simulates the forward (direct) rigi-boddy dynamics of the robot model SerialLink.
	// Given a joint torque vector, this block returns joint positions (q), velocities (qd) and accelerations (qdd).
	//
	// This block has one regular input, the joint torque vector.
	//
	// The number of output of this block is three. The first output represents the joint state vector q.
	// The second output is the joint velocity qd. The third output represents the joint acceleration vector qdd.
	// 
	// Each of these vector valued signals must have n elements, where n is the number of SerialLink's constituent links.
	// The robot model SerialLink can be specified as block parameter.
	//
	// <refsection>
    // <title>Dialog Box</title>
    // <variablelist>
    //   <varlistentry><term>Robot object    :</term>
    // 	<listitem><para>SerialLink. It represents the robot model to be simulated. Note that this variable is a symbolic parameter which has to be defined in the context of the Xcos diagram in order to be used.</para></listitem></varlistentry>
    //   <varlistentry><term>Initial joint state vector    :</term>
    // 	<listitem><para>n-element vector. It descries the initial joint coordinates, i.e. the initial configuration of manipulator. Initial value: 0.</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
	//
	// See also
	// accel_if
	// rne_if
	// fdyn
	//
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci
	
    x = [];
    y = [];
    typ = [];
	style = 'blockWithLabel;fillColor=red;displayedLabel=Robot;fontStyle=1;fontSize=12';
	in_label = ['Q']; 
	out_label = ['q';'qd';'qdd'];
    select job,
        case "set" then
            graphics = arg1.graphics;
            exprs = graphics.exprs;
            model = arg1.model;
            while %T do,
                [ok, robotName, q0, exprs] = scicos_getvalue(..
                    "Rigid-body dynamic model of robot",..
                    ["Robot object";..
                    "Initial joint state vector"],..
                    list("str", -1, "vec", -1),..
                    exprs);
                    robot = evstr(robotName);
                q0 = q0(:);
                if ok & typeof(robot) ~= "Robot" then
                    // not a robot
                    messagebox("invalid block parameter: required a robot object");
                    ok = %F;
                else
                    if ok & robot.n == 0 then
                        // a robot model to be simulated still must be specified
                        ok = %F;
                    else
                        if ok & size(q0, "*") ~= robot.n then
                            messagebox("invalid block parameter: initial joint state vector must have length " + sci2exp(robot.n));
                            ok = %F;
                        end
                    end
                end
                if ~ok then
                    break;
                end
                [model, graphics, ok] = set_io(model, graphics, list([robot.n, 1], 1), list([robot.n, 1; robot.n, 1; robot.n, 1], [1; 1; 1]), [], []);
                if ok then
                    // paths to updatable parameters or states
                    ppath = list(1, 2, 3);
                    newpar = list();
                    y = 0;
                    needcompile = 0;
                    for path = ppath do
                        np = size(path, "*");
                        spath = list();
                        for k = 1:np,
                            spath($+1) = "model";
                            spath($+1) = "rpar";
                            spath($+1) = "objs";
                            spath($+1) = path(k);
                        end
                        // get the block
                        xx = arg1(spath); // xx = arg1.model.rpar.objs(1)
                        xxn = xx;

                        // execute its interfacing function
                        graphics1 = xx.graphics;
                        exprs1 = graphics1.exprs;
                        model1 = xx.model;
                        if path == 2 then

                            // case "accel_if"
                            exprs1 = exprs(1);
                            [model1, graphics1, ok] = set_io(model1, graphics1, list([robot.n, 1; robot.n, 1; robot.n, 1], [1; 1; 1]), list([robot.n, 1], 1), [], []);
                            if ok then
                                model1.opar = list(robot.links, robot.gravity, robot.offset, robot.mdh);
                                graphics1.exprs = exprs1;
                                graphics1.gr_i(1) = "xstringb(orig(1), orig(2), [""accel""; """"; """ + robot.name + """], sz(1), sz(2), ""fill"");";
                                xxn.graphics = graphics1;
                                xxn.model = model1;
                            end

                        else

                            // case "INTEGRAL"
                            if path == 1 then
                                // accel2velocity integrator
                                x0 = zeros(robot.n, 1);
                            else
                                // velocity2position integrator
                                x0 = q0;
                            end
                            reinit = 0;
                            satur = 0;
                            maxp = 1;
                            lowp = -1;
                            exprs1 = [sci2exp(x0); sci2exp(reinit); sci2exp(satur); sci2exp(maxp); sci2exp(lowp)];
                            rpar=[maxp; lowp];
                            model1.rpar = rpar;
                            model1.state = x0;
                            model1.nzcross = 0;
                            model1.nmode = 0;
                            [model1, graphics1, ok] = set_io(model1, graphics1, list([length(x0), 1], 1), list([length(x0), 1], 1), [], []);
                            if ok then
                                graphics1.exprs = exprs1;
                                xxn.graphics = graphics1;
                                xxn.model = model1;
                            end

                        end

                        // set needcompile
                        if diffobjs(xxn, xx) then
                            model1 = xx.model;
                            model1_n = xxn.model;
                            if or(model1.in <> model1_n.in) | or(model1.out <> model1_n.out) then
                                needcompile = 1;
                            end
                            if (size(model1.in, "*") <> size(model1_n.in, "*")) |..
                               (size(model1.out, "*") <> size(model1_n.out, "*")) then
                                needcompile = 4;
                            end
                            if model1.sim == "input" | model1.sim == "output" then
                                if model1.ipar <> model1_n.ipar then
                                    needcompile = 4;
                                end
                            end
                        end

                        // parameter or states changed
                        arg1(spath) = xxn;                  // Update
                        newpar(size(newpar) + 1) = path;    // Notify modification
                        y = max(y, needcompile);            // Notify needcompile
                    end

                    // Update x
                    arg1.graphics.exprs = exprs;
                    //arg1.graphics.gr_i(1) = "xstringb(orig(1), orig(2), [""SerialLink""; """ + robot.name + """], sz(1), sz(2), ""fill"");";
					arg1.graphics.style = 'blockWithLabel;fillColor=red;displayedLabel=Robot <BR><BR> %1$s;fontStyle=1;fontSize=12';
                    x = arg1;
                    typ = newpar;
                    break
                end
            end

        case "define" then
scs_m_1=scicos_diagram(..
        version="scicos4.2",..
        props=scicos_params(..
              wpar=[64.374359,806.12085,45.360505,543.82031,1016,690,0,22,1016,690,0,7,1.4],..
              Title="SerialLink_if",..
              tol=[0.0001,0.000001,1.000E-10,100001,0,0],..
              tf=100000,..
              context=" ",..
              void1=[],..
              options=tlist(["scsopt","3D","Background","Link","ID","Cmap"],list(%t,33),[8,1],[1,5],..
              list([5,1],[4,1]),[0.8,0.8,0.8]),..
              void2=[],..
              void3=[],..
              doc=list()))
scs_m_1.objs(1)=scicos_block(..
                gui="INTEGRAL_m",..
                graphics=scicos_graphics(..
                         orig=[410.67459,360.59844],..
                         sz=[40,40],..
                         flip=%t,..
                         theta=0,..
                         exprs=["0";"0";"0";"1";"-1"],..
                         pin=22,..
                         pout=12,..
                         pein=[],..
                         peout=[],..
                         gr_i=list(..
                         ["thick=xget(''thickness'')";
                         "pat=xget(''pattern'')";
                         "fnt=xget(''font'')";
                         "xpoly(orig(1)+[0.7;0.62;0.549;0.44;0.364;0.291]*sz(1),orig(2)+[0.947;0.947;0.884;0.321;0.255;0.255]*sz(2),''lines'')";
                         "xset(''thickness'',thick)";
                         "xset(''pattern'',pat)";
                         "xset(''font'',fnt(1),fnt(2))"],8),..
                         id="1/s",..
                         in_implicit="E",..
                         out_implicit="E"),..
                model=scicos_model(..
                         sim=list("integral_func",4),..
                         in=1,..
                         in2=1,..
                         intyp=1,..
                         out=1,..
                         out2=1,..
                         outtyp=1,..
                         evtin=[],..
                         evtout=[],..
                         state=0,..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=[],..
                         opar=list(),..
                         blocktype="c",..
                         firing=[],..
                         dep_ut=[%f,%t],..
                         label="acc2vel",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(2)=scicos_block(..
                gui="accel_if",..
                graphics=scicos_graphics(..
                         orig=[290.45367,360.59844],..
                         sz=[60,40],..
                         flip=%t,..
                         theta=0,..
                         exprs="SerialLink()",..
                         pin=[11;18;19],..
                         pout=20,..
                         pein=[],..
                         peout=[],..
                         gr_i=list(..
                         "xstringb(orig(1), orig(2), [""accel""; """"; ""noname""], sz(1), sz(2), ""fill"");",..
                         8),..
                         id="",..
                         in_implicit=["E";"E";"E"],..
                         out_implicit="E"),..
                model=scicos_model(..
                         sim=list("accel_cf",5),..
                         in=[-1;-1;-1],..
                         in2=[1;1;1],..
                         intyp=[1;1;1],..
                         out=-1,..
                         out2=1,..
                         outtyp=1,..
                         evtin=[],..
                         evtout=[],..
                         state=[],..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=[],..
                         opar=list(),..
                         blocktype="d",..
                         firing=[],..
                         dep_ut=[%t,%f],..
                         label="",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(3)=scicos_block(..
                gui="INTEGRAL_m",..
                graphics=scicos_graphics(..
                         orig=[510.44434,360.59844],..
                         sz=[40,40],..
                         flip=%t,..
                         theta=0,..
                         exprs=["0";"0";"0";"1";"-1"],..
                         pin=14,..
                         pout=8,..
                         pein=[],..
                         peout=[],..
                         gr_i=list(..
                         ["thick=xget(''thickness'')";
                         "pat=xget(''pattern'')";
                         "fnt=xget(''font'')";
                         "xpoly(orig(1)+[0.7;0.62;0.549;0.44;0.364;0.291]*sz(1),orig(2)+[0.947;0.947;0.884;0.321;0.255;0.255]*sz(2),''lines'')";
                         "xset(''thickness'',thick)";
                         "xset(''pattern'',pat)";
                         "xset(''font'',fnt(1),fnt(2))"],8),..
                         id="1/s",..
                         in_implicit="E",..
                         out_implicit="E"),..
                model=scicos_model(..
                         sim=list("integral_func",4),..
                         in=1,..
                         in2=1,..
                         intyp=1,..
                         out=1,..
                         out2=1,..
                         outtyp=1,..
                         evtin=[],..
                         evtout=[],..
                         state=0,..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=[],..
                         opar=list(),..
                         blocktype="c",..
                         firing=[],..
                         dep_ut=[%f,%t],..
                         label="vel2pos",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(4)=scicos_block(..
                gui="IN_f",..
                graphics=scicos_graphics(..
                         orig=[197.34825,360.59844],..
                         sz=[20,20],..
                         flip=%t,..
                         theta=0,..
                         exprs="1",..
                         pin=[],..
                         pout=19,..
                         pein=[],..
                         peout=[],..
                         gr_i=list(" ",8),..
                         id="",..
                         in_implicit=[],..
                         out_implicit="E"),..
                model=scicos_model(..
                         sim="input",..
                         in=[],..
                         in2=[],..
                         intyp=1,..
                         out=-1,..
                         out2=-2,..
                         outtyp=-1,..
                         evtin=[],..
                         evtout=[],..
                         state=[],..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=1,..
                         opar=list(),..
                         blocktype="c",..
                         firing=[],..
                         dep_ut=[%f,%f],..
                         label="",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(5)=scicos_block(..
                gui="OUT_f",..
                graphics=scicos_graphics(..
                         orig=[619.11369,370.59844],..
                         sz=[20,20],..
                         flip=%t,..
                         theta=0,..
                         exprs="1",..
                         pin=10,..
                         pout=[],..
                         pein=[],..
                         peout=[],..
                         gr_i=list(" ",8),..
                         id="",..
                         in_implicit="E",..
                         out_implicit=[]),..
                model=scicos_model(..
                         sim="output",..
                         in=-1,..
                         in2=-2,..
                         intyp=-1,..
                         out=[],..
                         out2=[],..
                         outtyp=1,..
                         evtin=[],..
                         evtout=[],..
                         state=[],..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=1,..
                         opar=list(),..
                         blocktype="c",..
                         firing=[],..
                         dep_ut=[%f,%f],..
                         label="",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(6)=scicos_block(..
                gui="OUT_f",..
                graphics=scicos_graphics(..
                         orig=[619.11369,289.92731],..
                         sz=[20,20],..
                         flip=%t,..
                         theta=0,..
                         exprs="2",..
                         pin=15,..
                         pout=[],..
                         pein=[],..
                         peout=[],..
                         gr_i=list(" ",8),..
                         id="",..
                         in_implicit="E",..
                         out_implicit=[]),..
                model=scicos_model(..
                         sim="output",..
                         in=-1,..
                         in2=-2,..
                         intyp=-1,..
                         out=[],..
                         out2=[],..
                         outtyp=1,..
                         evtin=[],..
                         evtout=[],..
                         state=[],..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=2,..
                         opar=list(),..
                         blocktype="c",..
                         firing=[],..
                         dep_ut=[%f,%f],..
                         label="",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(7)=scicos_block(..
                gui="OUT_f",..
                graphics=scicos_graphics(..
                         orig=[619.11369,209.83962],..
                         sz=[20,20],..
                         flip=%t,..
                         theta=0,..
                         exprs="3",..
                         pin=23,..
                         pout=[],..
                         pein=[],..
                         peout=[],..
                         gr_i=list(" ",8),..
                         id="",..
                         in_implicit="E",..
                         out_implicit=[]),..
                model=scicos_model(..
                         sim="output",..
                         in=-1,..
                         in2=-2,..
                         intyp=-1,..
                         out=[],..
                         out2=[],..
                         outtyp=1,..
                         evtin=[],..
                         evtout=[],..
                         state=[],..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=3,..
                         opar=list(),..
                         blocktype="c",..
                         firing=[],..
                         dep_ut=[%f,%f],..
                         label="",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(8)=scicos_link(..
                  xx=[559.01577;579.80056],..
                  yy=[380.59844;380.59844],..
                  id="drawlink",..
                  thick=[0,0],..
                  ct=[1,1],..
                  from=[3,1,0],..
                  to=[9,1,1])
scs_m_1.objs(9)=scicos_block(..
                gui="SPLIT_f",..
                graphics=scicos_graphics(..
                         orig=[579.80056;380.59844],..
                         sz=[0.3333333,0.3333333],..
                         flip=%t,..
                         theta=0,..
                         exprs=[],..
                         pin=8,..
                         pout=[10;11],..
                         pein=[],..
                         peout=[],..
                         gr_i=list([],8),..
                         id="",..
                         in_implicit="E",..
                         out_implicit=["E";"E";"E"]),..
                model=scicos_model(..
                         sim="lsplit",..
                         in=-1,..
                         in2=[],..
                         intyp=1,..
                         out=[-1;-1;-1],..
                         out2=[],..
                         outtyp=1,..
                         evtin=[],..
                         evtout=[],..
                         state=[],..
                         dstate=[],..
                         odstate=list(),..
                         rpar=[],..
                         ipar=[],..
                         opar=list(),..
                         blocktype="c",..
                         firing=[],..
                         dep_ut=[%t,%f],..
                         label="",..
                         nzcross=0,..
                         nmode=0,..
                         equations=list()),..
                doc=list())
scs_m_1.objs(10)=scicos_link(..
                   xx=[579.80056;619.11369],..
                   yy=[380.59844;380.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[9,1,0],..
                   to=[5,1,1])
scs_m_1.objs(11)=scicos_link(..
                   xx=[579.80056;579.80056;260.0319;260.0319;281.88224],..
                   yy=[380.59844;500.47598;500.47598;390.59844;390.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[9,2,0],..
                   to=[2,1,1])
scs_m_1.objs(12)=scicos_link(..
                   xx=[459.24602;479.7816],..
                   yy=[380.59844;380.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[1,1,0],..
                   to=[16,1,1])
scs_m_1.objs(13)=scicos_block(..
                 gui="SPLIT_f",..
                 graphics=scicos_graphics(..
                          orig=[479.7816;380.59844],..
                          sz=[0.3333333,0.3333333],..
                          flip=%t,..
                          theta=0,..
                          exprs=[],..
                          pin=17,..
                          pout=[14;15],..
                          pein=[],..
                          peout=[],..
                          gr_i=list([],8),..
                          id="",..
                          in_implicit="E",..
                          out_implicit=["E";"E";"E"]),..
                 model=scicos_model(..
                          sim="lsplit",..
                          in=-1,..
                          in2=[],..
                          intyp=1,..
                          out=[-1;-1;-1],..
                          out2=[],..
                          outtyp=1,..
                          evtin=[],..
                          evtout=[],..
                          state=[],..
                          dstate=[],..
                          odstate=list(),..
                          rpar=[],..
                          ipar=[],..
                          opar=list(),..
                          blocktype="c",..
                          firing=[],..
                          dep_ut=[%t,%f],..
                          label="",..
                          nzcross=0,..
                          nmode=0,..
                          equations=list()),..
                 doc=list())
scs_m_1.objs(14)=scicos_link(..
                   xx=[479.7816;501.87292],..
                   yy=[380.59844;380.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[13,1,0],..
                   to=[3,1,1])
scs_m_1.objs(15)=scicos_link(..
                   xx=[479.7816;479.7816;619.11369],..
                   yy=[380.59844;299.92731;299.92731],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[13,2,0],..
                   to=[6,1,1])
scs_m_1.objs(16)=scicos_block(..
                 gui="SPLIT_f",..
                 graphics=scicos_graphics(..
                          orig=[479.7816;380.59844],..
                          sz=[0.3333333,0.3333333],..
                          flip=%t,..
                          theta=0,..
                          exprs=[],..
                          pin=12,..
                          pout=[17;18],..
                          pein=[],..
                          peout=[],..
                          gr_i=list([],8),..
                          id="",..
                          in_implicit="E",..
                          out_implicit=["E";"E";"E"]),..
                 model=scicos_model(..
                          sim="lsplit",..
                          in=-1,..
                          in2=[],..
                          intyp=1,..
                          out=[-1;-1;-1],..
                          out2=[],..
                          outtyp=1,..
                          evtin=[],..
                          evtout=[],..
                          state=[],..
                          dstate=[],..
                          odstate=list(),..
                          rpar=[],..
                          ipar=[],..
                          opar=list(),..
                          blocktype="c",..
                          firing=[],..
                          dep_ut=[%t,%f],..
                          label="",..
                          nzcross=0,..
                          nmode=0,..
                          equations=list()),..
                 doc=list())
scs_m_1.objs(17)=scicos_link(..
                   xx=[479.7816;479.7816],..
                   yy=[380.59844;380.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[16,1,0],..
                   to=[13,1,1])
scs_m_1.objs(18)=scicos_link(..
                   xx=[479.7816;479.7816;239.59007;239.59007;281.88224],..
                   yy=[380.59844;439.79392;439.79392;380.59844;380.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[16,2,0],..
                   to=[2,2,1])
scs_m_1.objs(19)=scicos_link(..
                   xx=[217.34825;281.88224],..
                   yy=[370.59844;370.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[4,1,0],..
                   to=[2,3,1])
scs_m_1.objs(20)=scicos_link(..
                   xx=[359.0251;379.76263],..
                   yy=[380.59844;380.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[2,1,0],..
                   to=[21,1,1])
scs_m_1.objs(21)=scicos_block(..
                 gui="SPLIT_f",..
                 graphics=scicos_graphics(..
                          orig=[379.76263;380.59844],..
                          sz=[0.3333333,0.3333333],..
                          flip=%t,..
                          theta=0,..
                          exprs=[],..
                          pin=20,..
                          pout=[22;23],..
                          pein=[],..
                          peout=[],..
                          gr_i=list([],8),..
                          id="",..
                          in_implicit="E",..
                          out_implicit=["E";"E";"E"]),..
                 model=scicos_model(..
                          sim="lsplit",..
                          in=-1,..
                          in2=[],..
                          intyp=1,..
                          out=[-1;-1;-1],..
                          out2=[],..
                          outtyp=1,..
                          evtin=[],..
                          evtout=[],..
                          state=[],..
                          dstate=[],..
                          odstate=list(),..
                          rpar=[],..
                          ipar=[],..
                          opar=list(),..
                          blocktype="c",..
                          firing=[],..
                          dep_ut=[%t,%f],..
                          label="",..
                          nzcross=0,..
                          nmode=0,..
                          equations=list()),..
                 doc=list())
scs_m_1.objs(22)=scicos_link(..
                   xx=[379.76263;402.10316],..
                   yy=[380.59844;380.59844],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[21,1,0],..
                   to=[1,1,1])
scs_m_1.objs(23)=scicos_link(..
                   xx=[379.76263;379.76263;619.11369],..
                   yy=[380.59844;219.83962;219.83962],..
                   id="drawlink",..
                   thick=[0,0],..
                   ct=[1,1],..
                   from=[21,2,0],..
                   to=[7,1,1])
model=scicos_model(..
         sim="csuper",..
         in=-1,..
         in2=1,..
         intyp=1,..
         out=[-1;-1;-1],..
         out2=[1;1;1],..
         outtyp=[1;1;1],..
         evtin=[],..
         evtout=[],..
         state=[],..
         dstate=[],..
         odstate=list(),..
         rpar=scs_m_1,..
         ipar=[],..
         opar=list(),..
         blocktype="h",..
         firing=[],..
         dep_ut=[%f,%f],..
         label="",..
         nzcross=0,..
         nmode=0,..
         equations=list())

            q0 = [0 0 0 0 0 0];
            exprs = ["robot"; sci2exp(q0)];
            x=standard_define([4 4],model,exprs,"");
			x.graphics.style = style;
			x.graphics.out_label = out_label;
			x.graphics.in_label = in_label;
    end

endfunction
