//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function T = rpy2tr(roll, varargin)
    // Roll-pitch-yaw angles to homogeneous transform
    //
    // Calling Sequence
    // T = rpy2tr(RPY, OPTIONS)
    // T = rpy2tr(ROLL, PITCH, YAW, OPTIONS)
    //
    // Description
    // T = RPY2TR(RPY, OPTIONS) is a homogeneous transformation equivalent to the 
    // specified roll, pitch, yaw angles which correspond to rotations about the 
    // X, Y, Z axes respectively. If RPY has multiple rows they are assumed to 
    // represent a trajectory and T is a three dimensional matrix, where the last index  
    // corresponds to the rows of RPY.
    //
    // T = RPY2TR(ROLL, PITCH, YAW, OPTIONS) as above but the roll-pitch-yaw angles 
    // are passed as separate arguments. If ROLL, PITCH and YAW are column vectors 
    // they are assumed to represent a trajectory and T is a three dimensional matrix,
    // where the last index corresponds to the rows of ROLL, PITCH, YAW.
    //
    // <refsection>
    // <title>Options</title>
    // <variablelist>
    //   <varlistentry><term>'deg'     :</term>
    // 	<listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    //   <varlistentry><term>'zyx'     :</term>
    // 	<listitem><para>Return solution for sequential rotations about Z, Y, X axes (Paul book)</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para> In previous releases (8) the angles corresponded to rotations about ZYX. Many 
    //   texts (Paul, Spong) use the rotation order ZYX. This old behaviour can be enabled 
    //   by passing the option 'zyx' </para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // tr2rpy 
    // rpy2r 
    // eul2tr
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    R = rpy2r(roll, varargin(:));
    T = r2t(R);

endfunction
