//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.


function R = t2r(T)
    // Return rotational submatrix of a homogeneous transformation
    //
    // Calling Sequence
    //  R = t2r(T)
    //
    // Description
    // R = T2R(T) is the orthonormal rotation matrix component of homogeneous 
    // transformation matrix T.
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para> Works for T in SE(2) or SE(3) </para></listitem>
    // <listitem><para> If T is 4x4, then R is 3x3. </para></listitem>
    // <listitem><para> If T is 3x3, then R is  2x2. </para></listitem>
    // <listitem><para> The validity of rotational part is not checked </para></listitem>
    // <listitem><para> For a homogeneous transform sequence returns a rotation matrix sequence </para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // r2t
    // rt2tr 
    // tr2rt
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    // check dimensions: T is SE(2) or SE(3)
    d = size(T);
    if d(1) ~= d(2)
        error('t2r: badarg: matrix must be square');
    end
    if ~or(d(1) == [3 4])
        error('t2r: badarg: argument is not a homogeneous transform (sequence)');
    end

    n = d(1);     //works for SE(2) or SE(3)

    if length(d) == 2
        // single matrix case
        R = T(1:n-1,1:n-1);
    else
        //  matrix sequence case
        R = zeros(3,3,d(3));
        for i=1:d(3)
            R(:,:,i) = T(1:n-1,1:n-1,i);
        end
    end
endfunction
