//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function handle = teach(r,varargin)
    // Graphical teach pendant
    //
    // Calling Sequence
    // R.teach()
    // teach(R)
    //
    // Description
    // R.teach() drive a graphical robot by means of a graphical slider panel.
    // If no graphical robot exists one is created in a new window.  Otherwise
    // all current instances of the graphical robot are driven.
    //
    // See also 
    // SerialLink
    // plot_robot 
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    [%nargout, %nargin] = argn(0);
    bgcol = [135 206 250]/255;
    yellow = [250 250 100]/255;

    n = r.n;
    width = 300;
    height = 40;

    minVal = -%pi;
    maxVal = %pi;
    scale = 100/%pi*ones(n, 1);
    q_scale = ones(n, 1);
    rplot_id = [];
    h_t=[];

    qlim = r.qlim;
    if isempty(qlim),
        qlim = [minVal*ones(n, 1) maxVal*ones(n, 1)];
    end
    q = zeros(1,n);
    //.....
    q_scale = ones(r.n,1);
    for i=1:n
        L=r.links(i);
        //            if opt.degrees & L.isrevolute
        //            q_scale(i) = 180/pi;
        //           end
    end
    handles.q_scale = q_scale;
    handles.t6 =[];
    //handles.orientation = opt.orientation;

    T6 = fkine(r,q);//r.fkine(q);

    fig = figure('Position', [0 -height+40 width height*(n+2)+80]);
    //delete( get(fig, 'Children') )
    delmenu(fig.figure_id,gettext("&File")); 
    delmenu(fig.figure_id,gettext("&Tools")); 
    delmenu(fig.figure_id,gettext("&Edit")); 
    delmenu(fig.figure_id,gettext("&?")); 
    toolbar(fig.figure_id,"off"); 

    // first we check to see if there are any graphical robots of
    // this name, if so we use them, otherwise create a robot plot.
    rh = rt_findtag(r.name);
    // attempt to get current joint config of graphical robot

    if ~isempty(rh)
        ud = rh(1).user_data;
        r = ud(2);
        if %nargin < 2 then
            q = r.q.';
        else
            q = b;
            plot_robot(r, q);
        end
        rplot_id = r.handles.links.parent.parent.figure_id;
    else
        cfhdb = scf();
        drawlater();
        rplot_id = cfhdb.figure_id;
        cfhdb.pixmap = "on";
        cfhdb.children.tight_limits = "on";
//        cfhdb.children.rotation_angles = [80, -46];
        cfhdb.tag = "Robot"
        if %nargin < 2 then
            q = zeros(1, n);
        else
            q = b;
        end
        plot_robot(r, q);

        drawnow();
    end
    
    // robot name text box
    h_whichrobot = uicontrol(fig, "Style", "text",..
    "Units", "pixels",..
    "fontsize", 20,..
    "Horizontalalignment", "center",..
    "Position", [0 height*(n+1.2)+40 0.8*width 0.8*height],..
    "BackgroundColor", [1, 1, 1],..
    "Tag", r.name,..                            // useful var when executing callbacks
    "Userdata", [scale; q_scale; rplot_id],..    // another useful var
    "String", r.name);                          // last useful var

    // now make the sliders
    for i=1:n
        // slider label
        uicontrol(fig, ..
        'Style', 'text', ...
        'Units', 'pixels', ...
        'BackgroundColor', bgcol, ...
        'Position', [0 height*(n-i)+40 width*0.1 height*0.4], ...
        'String', 'q'+sci2exp(i));

        // slider itself
        //q(i) = 0;//max( qlim(i,1), min( qlim(i,2), q(i))); // clip to range
        h_slider(i) = uicontrol(fig, ..
        'Style', 'slider', ...
        'Units', 'pixels', ...
        'Position', [width*0.1 height*(n-i)+40 width*0.7 height*0.4], ...
        'Min', scale(i)*qlim(i,1), ...
        'Max', scale(i)*qlim(i,2), ...
        'Value', scale(i)*q(i), ...
        'Tag', 'Slider'+sci2exp(i));


        // text box showing slider value, also editable
        h_edit(i) = uicontrol(fig, ..
        'Style', 'edit', ...
        'Units', 'pixels', ...
        'Position', [width*0.8 height*(n-i-0.1)+40 width*0.2 height*0.7], ...
        'String', sci2exp(q_scale(i)*q(i)), ...
        'Tag', 'Edit'+sci2exp(i));
    end
    handles.slider = h_slider;
    handles.edit = h_edit;

    // set(handles.slider(1))

    // X
    uicontrol(fig, 'Style', 'text', ...
    'Units', 'pixels', ...
    'Position', [0 height*(n+0.5)+40 0.06*width height/2], ...
    'BackgroundColor', yellow, ...
    'FontSize', 10, ...
    'HorizontalAlignment', 'right', ...
    'String', 'x:');

    h_t(1) = uicontrol(fig, 'Style', 'edit', ...
    'Units', 'pixels', ...
    'Position', [0.06*width height*(n+0.5)+40 width*0.2 height*0.6], ...
    'String', sprintf('%.3f', T6(2,4)), ...
    'Tag', 'hxyzatext'+ sci2exp(11));

    // Y
    uicontrol(fig, 'Style', 'text', ...
    'Units', 'pixels', ...
    'Position', [0.26*width height*(n+0.5)+40 0.06*width height/2], ...
    'BackgroundColor', yellow, ...
    'FontSize', 10, ...
    'HorizontalAlignment', 'right', ...
    'String', 'y:');

    h_t(2, 1) = uicontrol(fig, 'Style', 'edit', ...
    'Units', 'pixels', ...
    'Position', [0.32*width height*(n+0.5)+40 width*0.2 height*0.6], ...
    'Tag', 'hxyzatext' + sci2exp(21),...
    'String', sprintf('%.3f', T6(2,4)));

    // Z
    uicontrol(fig, 'Style', 'text', ...
    'Units', 'pixels', ...
    'Position', [0.52*width height*(n+0.5)+40 0.06*width height/2], ...
    'BackgroundColor', yellow, ...
    'FontSize', 10, ...
    'HorizontalAlignment', 'right', ...
    'String', 'z:');

    h_t(3, 1) = uicontrol(fig, 'Style', 'edit', ...
    'Units', 'pixels', ...
    'Position', [0.58*width height*(n+0.5)+40 width*0.2 height*0.6], ...
    'Tag', 'hxyzatext' + sci2exp(31),...
    'String', sprintf('%.3f', T6(3,4)));

    // Orientation
    //    switch opt.orientation
    //        case 'approach'
    labels = ['ax:', 'ay:', 'az:'];
    //        case 'eul'
    //            labels = {'\phi:', '\theta:', '\psi:'};
    //        case'rpy'
    //            labels = {'R:', 'P:', 'Y:'};
    //    end
    // AX
    uicontrol(fig, 'Style', 'text', ...
    'Units', 'pixels', ...
    'Position', [0 height*(n)+40 0.06*width height/2], ...
    'BackgroundColor', yellow, ...//'yellow',
    'FontSize', 10, ...
    'HorizontalAlignment', 'right', ...
    'String', labels(1));

    h_t(1,2) = uicontrol(fig, 'Style', 'edit', ...
    'Units', 'pixels', ...
    'Position', [0.06*width height*(n)+40 width*0.2 height*0.6], ...
    'Tag', 'hxyzatext' + sci2exp(12),...
    'String', sprintf('%.3f', T6(1,3)));

    // AY
    uicontrol(fig, 'Style', 'text', ...
    'Units', 'pixels', ...
    'Position', [0.26*width height*(n)+40 0.06*width height/2], ...
    'BackgroundColor', yellow, ...
    'FontSize', 10, ...
    'HorizontalAlignment', 'right', ...
    'String', labels(2));

    h_t(2,2) = uicontrol(fig, 'Style', 'edit', ...
    'Units', 'pixels', ...
    'Position', [0.32*width height*(n)+40 width*0.2 height*0.6], ...
    'Tag', 'hxyzatext' + sci2exp(22),...
    'String', sprintf('%.3f', T6(2,3)));

    // AZ
    uicontrol(fig, 'Style', 'text', ...
    'Units', 'pixels', ...
    'Position', [0.52*width height*(n)+40 0.06*width height/2], ...
    'BackgroundColor', yellow, ...
    'FontSize', 10, ...
    'HorizontalAlignment', 'right', ...
    'String', labels(3));

    h_t(3, 2) = uicontrol(fig, 'Style', 'edit', ...
    'Units', 'pixels', ...
    'Position', [0.58*width height*(n)+40 width*0.2 height*0.6], ...
    'Tag', 'hxyzatext' + sci2exp(32),...
    'String', sprintf('%.3f', T6(3,3)));


    // add buttons
    goButton = uicontrol(fig, ...
    'Style', 'pushbutton', ...
    'Units', 'pixels', ...
    'FontSize', 15, ...
    'Position', [0.05*width 8 0.9*width 0.7*height], ...
    'BackgroundColor', [0,0,1], ...
    'String', 'record');
    set(goButton, "Callback", "teach_callback_2("+sci2exp(h_whichrobot.string) +")");


    quitButton = uicontrol(fig,...
    'Style', 'pushbutton', ...
    'Units', 'pixels', ...
    'FontSize', 16, ...
    'Position', [0.8*width height*n+40 0.2*width 2*height], ...
    'BackgroundColor', [1,0,0], ...
    'String', 'Quit');
    set(quitButton, "Callback", "teach_callback_3("+sci2exp(h_whichrobot.string) +")");


    // now assign the callbacks
    for i=1:n
        set(h_slider(i), "Callback", "teach_callback("+sci2exp(i)+","""+h_whichrobot.string+"'")");
    end

    if %nargout > 0
        handle = fig;
    end

endfunction
