//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function euler = tr2eul(m, varargin)
    // Convert homogeneous transform to Euler angles
    //
    // Calling Sequence
    // EUL = tr2eul(T, OPTIONS)
    // EUL = tr2eul(R, OPTIONS) 
    //
    // Description
    // EUL = TR2EUL(T, OPTIONS) are the ZYZ Euler angles expressed as a row vector
    // corresponding to the rotational part of a homogeneous transform T.
    // The 3 angles EUL=[PHI,THETA,PSI] correspond to sequential rotations about 
    // the Z, Y and Z axes respectively.
    //
    // EUL = TR2EUL(R, OPTIONS) are the ZYZ Euler angles expressed as a row vector
    // corresponding to the orthonormal rotation matrix R.
    //
    // If R or T represents a trajectory (has 3 dimensions), then each row of EUL
    // corresponds to a step of the trajectory.
    //
    // <refsection>
    // <title>OPTIONS</title>
    // <variablelist>
    //   <varlistentry><term>'deg'     :</term>
    // 	<listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>There is a singularity for the case where THETA=0 in which case 
    // PHI is arbitrarily set to zero and PSI is the sum (PHI+PSI).</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also  
    // eul2tr
    // tr2rpy
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    opt.deg = %f;
    if ~isempty(varargin) then
        if varargin(1) == 'deg' then
            opt.deg = %t;
        else
            disp('unknown argument');
        end
    end


    s = size(m);
    if length(s) > 2
        euler = zeros(s(3), 3);
        for i=1:s(3)
            euler(i,:) = tr2eul(m(:,:,i));
        end
        if opt.deg
            euler = euler * 180/%pi;
        end
        return
    end

    euler = zeros(1,3);

    //	 Method as per Paul, p 69.
    //	 phi = atan2(ay, ax)
    //	 Only positive phi is returned.
    if abs(m(1,3)) < %eps & abs(m(2,3)) < %eps
        // singularity
        euler(1) = 0;
        sp = 0;
        cp = 1;
        euler(2) = atan(cp*m(1,3) + sp*m(2,3), m(3,3));
        euler(3) = atan(-sp * m(1,1) + cp * m(2,1), -sp*m(1,2) + cp*m(2,2));
    else
        euler(1) = atan(m(2,3), m(1,3));
        sp = sin(euler(1));
        cp = cos(euler(1));
        euler(2) = atan(cp*m(1,3) + sp*m(2,3), m(3,3));
        euler(3) = atan(-sp * m(1,1) + cp * m(2,1), -sp*m(1,2) + cp*m(2,2));
    end
    if opt.deg
        euler = euler * 180/%pi;
    end
endfunction
