//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function To = trinterp(A, B, C)
    // Interpolate homogeneous transformations
    //
    // Calling Sequence
    // To = trinterp(T0, T1, S)
    // To = trinterp(To, S)
    //
    // Description
    // To = TRINTERP(T0, T1, S) is a homogeneous transform interpolation 
    // between T0 when S=0 to T1 when S=1.  Rotation is interpolated using 
    // quaternion spherical linear interpolation.  If S (Nx1) then To (4x4xN)
    // is a sequence of homogeneous transforms corresponding to the interpolation 
    // values in S.
    //
    // To = TRINTERP(To, S) is a transform that varies from the identity matrix when
    // S=0 to To when R=1.  If S (Nx1) then To (4x4xN) is a sequence of homogeneous 
    // transforms corresponding to the interpolation values in S.
    //
    // See also 
    // craj
    // Quaternion
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci

    [%nargout,%nargin]=argn(0);
    if %nargin == 3
        //	TR = TRINTERP(T0, T1, r)
        T0 = A; T1 = B; r = C;

        if length(r) > 1
            To = [];
            for rr=r(:)'
                TT = trinterp(T0, T1, rr);
                To = cat(3, To, TT);
            end
            return;
        end
        q0 = Quaternion(T0);
        q1 = Quaternion(T1);

        p0 = transl(T0);
        p1 = transl(T1);

        qrr = qinterp(q0,q1, r);
        pr = p0*(1-r) + r*p1;
    elseif %nargin == 2
        //	TR = TRINTERP(To, r)
        T0 = A; r = B;
        if length(r) > 1
            To = [];
            for rr=r(:)'
                TT = trinterp(T0, rr);
                To = cat(3, To, TT);
            end
            return;
        end
        q0 = Quaternion(T0);
        p0 = transl(T0);

        qrr = qscale(q0,r);
        pr = r*p0;
    else
        error('must be 2 or 3 arguments');
    end
    To = rt2tr(qrr.R, pr);
endfunction
