//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function v = vex(S)
    // Convert skew-symmetric matrix to vector
    //
    // Calling Sequence
    // V = vex(S)
    //
    // Description
    // V = VEX(S) is the vector (3x1) which has the skew-symmetric matrix S (3x3)
    //    
    // <latex>
    // \begin{bmatrix}
    // 0 & -vz & vy \\
    // vz & 0 & -vx \\
    // -vy & vx & 0 
    // \end{bmatrix}
    // </latex>
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>This is the inverse of the function SKEW().</para></listitem>
    // <listitem><para>No checking is done to ensure that the matrix is actually skew-symmetric.</para></listitem>
    // <listitem><para>The function takes the mean of the two elements that correspond
    // to each unique element of the matrix, ie. vx = 0.5*(S(3,2)-S(2,3)). </para></listitem>
    // </itemizedlist>
    // </refsection>
    // See also 
    // skew
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014 - Davide Cappucci



    if isrot(S) | ishomog(S)
        v = 0.5*[S(3,2)-S(2,3); S(1,3)-S(3,1); S(2,1)-S(1,2)];
    else
        error('argument must be a 3x3 matrix');
    end
endfunction

